@@ -23,6 +23,7 @@
 #include <linux/dma-mapping.h>
 #include <linux/pci.h>
 #include <linux/workqueue.h>
+#include <linux/wait.h>
 #include <linux/aer.h>
 #include <linux/interrupt.h>
 #include <linux/ethtool.h>
@@ -45,6 +46,12 @@
 #include <net/xdp_sock.h>
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
 #include <net/ipv6.h>
+#if IS_ENABLED(CONFIG_NET_DEVLINK)
+#include <net/devlink.h>
+#endif /* CONFIG_NET_DEVLINK */
+#if IS_ENABLED(CONFIG_DCB)
+#include <scsi/iscsi_proto.h>
+#endif /* CONFIG_DCB */
 #include "ice_devids.h"
 #include "ice_type.h"
 #include "ice_txrx.h"
@@ -59,10 +66,18 @@
 #include "virtchnl.h"
 #include "ice_virtchnl_pf.h"
 #include "ice_sriov.h"
+#include "ice_ptp.h"
+#include "ice_cgu.h"
+#include "ice_cgu_ops.h"
+#include "ice_cgu_util.h"
 #include "ice_fdir.h"
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
 #include "ice_xsk.h"
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
+#ifdef HAVE_NETDEV_UPPER_INFO
+#include "ice_lag.h"
+#endif /* HAVE_NETDEV_UPPER_INFO */
+#include "ice_trace.h"
 
 #if defined(HAVE_VXLAN_RX_OFFLOAD) || defined(HAVE_VXLAN_TYPE)
 #if IS_ENABLED(CONFIG_VXLAN)
@@ -90,6 +105,7 @@
 #include <linux/atomic.h>
 #include <linux/jiffies.h>
 #include "ice_arfs.h"
+#include "ice_vsi_vlan_ops.h"
 
 extern const char ice_drv_ver[];
 #define ICE_BAR0		0
@@ -102,11 +118,11 @@ extern const char ice_drv_ver[];
 #define ICE_MAX_NUM_DESC	8160
 #define ICE_DFLT_MIN_RX_DESC	512
 #ifdef HAVE_PF_RING
-#define ICE_DFLT_NUM_TX_DESC	4096
 #define ICE_DFLT_NUM_RX_DESC	4096
+#define ICE_DFLT_NUM_TX_DESC	4096
 #else
-#define ICE_DFLT_NUM_TX_DESC	256
 #define ICE_DFLT_NUM_RX_DESC	2048
+#define ICE_DFLT_NUM_TX_DESC	256
 #endif
 
 #define ICE_DFLT_TXQ_VMDQ_VSI	1
@@ -120,11 +136,20 @@ extern const char ice_drv_ver[];
 #endif
 #define ICE_DFLT_TRAFFIC_CLASS	BIT(0)
 #define ICE_INT_NAME_STR_LEN	(IFNAMSIZ + 16)
-#define ICE_AQ_LEN		64
+#ifdef FWLOG_SUPPORT
+/* if FW logging is on, then bump the admin q size to avoid overflows */
+#define ICE_AQ_LEN		1023
+#else
+#define ICE_AQ_LEN		192
+#endif /* FWLOG_SUPPORT */
 #define ICE_MBXSQ_LEN		64
-#define ICE_MBXRQ_LEN		1024
-#define ICE_MIN_MSIX		2
-#define ICE_FDIR_MSIX		1
+#define ICE_SBQ_LEN		64
+#define ICE_FDIR_MSIX		2
+#define ICE_MIN_LAN_MSIX	1
+#define ICE_OICR_MSIX		1
+#define ICE_RDMA_NUM_AEQ_MSIX	4
+#define ICE_MIN_RDMA_MSIX	2
+#define ICE_MIN_MSIX		(ICE_MIN_LAN_MSIX + ICE_OICR_MSIX)
 #define ICE_NO_VSI		0xffff
 #define ICE_VSI_MAP_CONTIG	0
 #define ICE_VSI_MAP_SCATTER	1
@@ -137,11 +162,13 @@ extern const char ice_drv_ver[];
 #define ICE_MAX_SMALL_RSS_QS	16
 #define ICE_RES_VALID_BIT	0x8000
 #define ICE_RES_MISC_VEC_ID	(ICE_RES_VALID_BIT - 1)
-#define ICE_RDMA_NUM_VECS	4
 #define ICE_RES_RDMA_VEC_ID	(ICE_RES_MISC_VEC_ID - 1)
+/* All VF control VSIs share the same irq, so assign a unique ID for them */
+#define ICE_RES_VF_CTRL_VEC_ID	(ICE_RES_RDMA_VEC_ID - 1)
 #define ICE_INVAL_Q_INDEX	0xffff
 #define ICE_INVAL_VFID		256
 
+#define ICE_MAX_RXQS_PER_TC		256	/* Used when setting VSI context per TC Rx queues */
 #define ICE_MAX_TXQS_PER_TC		8
 #define ICE_MAX_RDMA_QSET_PER_TC	1
 
@@ -150,6 +177,8 @@ extern const char ice_drv_ver[];
 
 #define ICE_MAX_RESET_WAIT		20
 
+#define ICE_VF_CHNL_START_TC		1
+
 #define ICE_VSIQF_HKEY_ARRAY_SIZE	((VSIQF_HKEY_MAX_INDEX + 1) *	4)
 
 #define ICE_DFLT_NETIF_M (NETIF_MSG_DRV | NETIF_MSG_PROBE | NETIF_MSG_LINK)
@@ -164,9 +193,9 @@ extern const char ice_drv_ver[];
 #define ICE_RX_DESC(R, i) (&(((union ice_32b_rx_flex_desc *)((R)->desc))[i]))
 #define ICE_TX_CTX_DESC(R, i) (&(((struct ice_tx_ctx_desc *)((R)->desc))[i]))
 #define ICE_TX_FDIRDESC(R, i) (&(((struct ice_fltr_desc *)((R)->desc))[i]))
-/* Default ports for tunnels when operating in Basic mode */
-#define ICE_DFLT_PORT_VXLAN		4789
-#define ICE_DFLT_PORT_GENEVE		6081
+
+#define ICE_ACL_ENTIRE_SLICE	1
+#define ICE_ACL_HALF_SLICE	2
 
 /* Minimum BW limit is 500 Kbps for any scheduler node */
 #define ICE_MIN_BW_LIMIT		500
@@ -199,13 +228,10 @@ extern const char ice_drv_ver[];
 #define ice_for_each_chnl_tc(i)	\
 	for ((i) = ICE_CHNL_START_TC; (i) < ICE_CHNL_MAX_TC; (i)++)
 
-#define ICE_UCAST_PROMISC_BITS (ICE_PROMISC_UCAST_TX | ICE_PROMISC_MCAST_TX | \
-				ICE_PROMISC_UCAST_RX | ICE_PROMISC_MCAST_RX)
+#define ICE_UCAST_PROMISC_BITS (ICE_PROMISC_UCAST_TX | ICE_PROMISC_UCAST_RX)
 
 #define ICE_UCAST_VLAN_PROMISC_BITS (ICE_PROMISC_UCAST_TX | \
-				     ICE_PROMISC_MCAST_TX | \
 				     ICE_PROMISC_UCAST_RX | \
-				     ICE_PROMISC_MCAST_RX | \
 				     ICE_PROMISC_VLAN_TX  | \
 				     ICE_PROMISC_VLAN_RX)
 
@@ -242,14 +268,12 @@ struct ice_channel {
 	u64 max_tx_rate;
 	u64 min_tx_rate;
 	atomic_t num_sb_fltr;
-#ifdef ADQ_PERF
 	/* counter index when side-band FD is used */
 	u32 fd_cnt_index;
 	/* queue used to setup inline-FD */
 	atomic_t fd_queue;
 	/* packets services thru' inline-FD filter */
 	u64 fd_pkt_cnt;
-#endif /* ADQ_PERF */
 	enum ice_channel_fltr_type fltr_type;
 	struct ice_vsi *ch_vsi;
 };
@@ -325,65 +349,77 @@ struct ice_sw {
 	u8 stat_offsets_loaded:1;
 };
 
-enum ice_state {
-	__ICE_TESTING,
-	__ICE_DOWN,
-	__ICE_NEEDS_RESTART,
-	__ICE_PREPARED_FOR_RESET,	/* set by driver when prepared */
-	__ICE_RESET_OICR_RECV,		/* set by driver after rcv reset OICR */
-	__ICE_DCBNL_DEVRESET,		/* set by dcbnl devreset */
-	__ICE_PFR_REQ,			/* set by driver and peers */
-	__ICE_CORER_REQ,		/* set by driver and peers */
-	__ICE_GLOBR_REQ,		/* set by driver and peers */
-	__ICE_CORER_RECV,		/* set by OICR handler */
-	__ICE_GLOBR_RECV,		/* set by OICR handler */
-	__ICE_EMPR_RECV,		/* set by OICR handler */
-	__ICE_SUSPENDED,		/* set on module remove path */
-	__ICE_RESET_FAILED,		/* set by reset/rebuild */
-	__ICE_RECOVERY_MODE,		/* set when recovery mode is detected */
-	__ICE_PREPPED_RECOVERY_MODE,	/* set on recovery mode transition */
+enum ice_pf_state {
+	ICE_TESTING,
+	ICE_DOWN,
+	ICE_NEEDS_RESTART,
+	ICE_PREPARED_FOR_RESET,	/* set by driver when prepared */
+	ICE_RESET_OICR_RECV,		/* set by driver after rcv reset OICR */
+	ICE_PFR_REQ,			/* set by driver and peers */
+	ICE_CORER_REQ,		/* set by driver and peers */
+	ICE_GLOBR_REQ,		/* set by driver and peers */
+	ICE_CORER_RECV,		/* set by OICR handler */
+	ICE_GLOBR_RECV,		/* set by OICR handler */
+	ICE_EMPR_RECV,		/* set by OICR handler */
+	ICE_SUSPENDED,		/* set on module remove path */
+	ICE_RESET_FAILED,		/* set by reset/rebuild */
+	ICE_RECOVERY_MODE,		/* set when recovery mode is detected */
+	ICE_PREPPED_RECOVERY_MODE,	/* set on recovery mode transition */
 	/* When checking for the PF to be in a nominal operating state, the
 	 * bits that are grouped at the beginning of the list need to be
-	 * checked. Bits occurring before __ICE_STATE_NOMINAL_CHECK_BITS will
+	 * checked. Bits occurring before ICE_STATE_NOMINAL_CHECK_BITS will
 	 * be checked. If you need to add a bit into consideration for nominal
 	 * operating state, it must be added before
-	 * __ICE_STATE_NOMINAL_CHECK_BITS. Do not move this entry's position
+	 * ICE_STATE_NOMINAL_CHECK_BITS. Do not move this entry's position
 	 * without appropriate consideration.
 	 */
-	__ICE_STATE_NOMINAL_CHECK_BITS,
-	__ICE_ADMINQ_EVENT_PENDING,
-	__ICE_MAILBOXQ_EVENT_PENDING,
-	__ICE_MDD_EVENT_PENDING,
-	__ICE_VFLR_EVENT_PENDING,
-	__ICE_FLTR_OVERFLOW_PROMISC,
-	__ICE_VF_DIS,
-	__ICE_CFG_BUSY,
-	__ICE_SERVICE_SCHED,
-	__ICE_SERVICE_DIS,
-	__ICE_FD_FLUSH_REQ,
-	__ICE_OICR_INTR_DIS,		/* Global OICR interrupt disabled */
-	__ICE_BAD_EEPROM,
-	__ICE_MDD_VF_PRINT_PENDING,	/* set when MDD event handle */
-	__ICE_STATE_NBITS		/* must be last */
+	ICE_STATE_NOMINAL_CHECK_BITS,
+	ICE_ADMINQ_EVENT_PENDING,
+	ICE_MAILBOXQ_EVENT_PENDING,
+	ICE_SIDEBANDQ_EVENT_PENDING,
+	ICE_MDD_EVENT_PENDING,
+	ICE_VFLR_EVENT_PENDING,
+	ICE_FLTR_OVERFLOW_PROMISC,
+	ICE_VF_DIS,
+	ICE_CFG_BUSY,
+	ICE_SERVICE_SCHED,
+	ICE_PTP_TX_TS_READY,
+	ICE_SERVICE_DIS,
+	ICE_FD_FLUSH_REQ,
+	ICE_OICR_INTR_DIS,		/* Global OICR interrupt disabled */
+	ICE_BAD_EEPROM,
+	ICE_MDD_VF_PRINT_PENDING,	/* set when MDD event handle */
+	ICE_VF_RESETS_DISABLED,	/* disable resets during ice_remove */
+	ICE_LINK_DEFAULT_OVERRIDE_PENDING,
+	ICE_PHY_INIT_COMPLETE,
+	ICE_FD_VF_FLUSH_CTX,		/* set at FD Rx IRQ or timeout */
+	ICE_STATE_NBITS		/* must be last */
 };
 
-enum ice_vsi_flags {
-	ICE_VSI_FLAG_UMAC_FLTR_CHANGED,
-	ICE_VSI_FLAG_MMAC_FLTR_CHANGED,
-	ICE_VSI_FLAG_VLAN_FLTR_CHANGED,
-	ICE_VSI_FLAG_PROMISC_CHANGED,
-	ICE_VSI_FLAG_NBITS		/* must be last */
+enum ice_vsi_state {
+	ICE_VSI_DOWN,
+	ICE_VSI_NEEDS_RESTART,
+	ICE_VSI_BUSY,
+	ICE_VSI_NETDEV_ALLOCD,
+	ICE_VSI_NETDEV_REGISTERED,
+	ICE_VSI_UMAC_FLTR_CHANGED,
+	ICE_VSI_MMAC_FLTR_CHANGED,
+	ICE_VSI_VLAN_FLTR_CHANGED,
+	ICE_VSI_PROMISC_CHANGED,
+	ICE_VSI_STATE_NBITS		/* must be last */
 };
 
 enum ice_chnl_feature {
-#ifdef ADQ_PERF
 	ICE_CHNL_FEATURE_FD_ENA, /* for side-band flow-director */
 	ICE_CHNL_FEATURE_INLINE_FD_ENA, /* for inline flow-director */
-#endif /* ADQ_PERF */
 	/* for pkt based inspection optimization - related to SW triggered
 	 * interrupt from napi_poll for channel enabled vector
 	 */
 	ICE_CHNL_FEATURE_PKT_INSPECT_OPT_ENA,
+	/* when set, allows cleaning of Rx queue(s) when napi_poll is invoked
+	 * due to busy_poll_stop
+	 */
+	ICE_CHNL_FEATURE_PKT_CLEAN_BP_STOP_ENA,
 	ICE_CHNL_FEATURE_NBITS		/* must be last */
 };
 
@@ -437,19 +473,21 @@ struct ice_tc_flower_action {
 	enum ice_sw_fwd_act_type fltr_act;
 };
 
-struct ice_tc_flower_vlan_hdrs {
+struct ice_tc_vlan_hdr {
 	__be16 vlan_id; /* Only last 12 bits valid */
 #ifdef HAVE_FLOW_DISSECTOR_VLAN_PRIO
 	u16 vlan_prio; /* Only last 3 bits valid (valid values: 0..7) */
 #endif
 };
 
-struct ice_tc_flower_lyr_2_4_hdrs {
+struct ice_tc_l2_hdr {
 	u8 dst_mac[ETH_ALEN];
 	u8 src_mac[ETH_ALEN];
-	struct ice_tc_flower_vlan_hdrs vlan_hdr;
-	__be16 dst_port;
-	__be16 src_port;
+	u16 n_proto;    /* Ethernet Protocol */
+};
+
+struct ice_tc_l3_hdr {
+	u8 ip_proto;    /* IPPROTO value */
 	union {
 		struct {
 			struct in_addr dst_ip;
@@ -466,11 +504,28 @@ struct ice_tc_flower_lyr_2_4_hdrs {
 #define src_ipv6_addr	ip.v6.src_ip6.s6_addr
 #define dst_ipv4	ip.v4.dst_ip.s_addr
 #define src_ipv4	ip.v4.src_ip.s_addr
-	u16 n_proto;    /* Ethernet Protocol */
-	u8 ip_proto;    /* IPPROTO value */
+
 	u8 tos;
 	u8 ttl;
+};
 
+struct ice_tc_l4_hdr {
+	__be16 dst_port;
+	__be16 src_port;
+};
+
+struct ice_tc_flower_lyr_2_4_hdrs {
+	/* L2 layer fields with their mask */
+	struct ice_tc_l2_hdr l2_key;
+	struct ice_tc_l2_hdr l2_mask;
+	struct ice_tc_vlan_hdr vlan_hdr;
+	/* L3 (IPv4[6]) layer fields with their mask */
+	struct ice_tc_l3_hdr l3_key;
+	struct ice_tc_l3_hdr l3_mask;
+
+	/* L4 layer fields with their mask */
+	struct ice_tc_l4_hdr l4_key;
+	struct ice_tc_l4_hdr l4_mask;
 };
 
 struct ice_tc_flower_fltr {
@@ -500,10 +555,14 @@ struct ice_tc_flower_fltr {
 #define ICE_TC_FLWR_TNL_TYPE_NONE        0xff
 	u8 tunnel_type;
 	struct ice_tc_flower_action	action;
+
+	/* cache ptr which is used wherever needed to communicate netlink
+	 * messages
+	 */
+	struct netlink_ext_ack *extack;
 };
 #endif /* HAVE_TC_SETUP_CLSFLOWER */
 
-#ifdef ADQ_PERF
 /* This is to be used only when channels are configured, to track state
  * at PF level, whether it should use RSS or inline flow-director and this
  * state gets set/reset appropriately as the HW flow-director table becomes
@@ -513,7 +572,6 @@ enum ice_advanced_state_t {
 	ICE_SWITCH_TO_RSS,
 	ICE_ADVANCED_STATE_LAST, /* this must be last */
 };
-#endif /* ADQ_PERF */
 
 /* struct that defines a VSI, associated with a dev */
 struct ice_vsi {
@@ -532,15 +590,18 @@ struct ice_vsi {
 	irqreturn_t (*irq_handler)(int irq, void *data);
 
 	u64 tx_linearize;
-	DECLARE_BITMAP(state, __ICE_STATE_NBITS);
-	DECLARE_BITMAP(flags, ICE_VSI_FLAG_NBITS);
+	DECLARE_BITMAP(state, ICE_VSI_STATE_NBITS);
 	unsigned int current_netdev_flags;
 	u32 tx_restart;
 	u32 tx_busy;
 	u32 rx_buf_failed;
 	u32 rx_page_failed;
-	int num_q_vectors;
-	int base_vector;		/* IRQ base for OS reserved vectors */
+	u32 rx_gro_dropped;
+#ifdef ICE_ADD_PROBES
+	u32 rx_page_reuse;
+#endif /* ICE_ADD_PROBES */
+	u16 num_q_vectors;
+	u16 base_vector;		/* IRQ base for OS reserved vectors */
 	enum ice_vsi_type type;
 	u16 vsi_num;			/* HW (absolute) index of this VSI */
 	u16 idx;			/* software index in pf->vsi[] */
@@ -560,16 +621,20 @@ struct ice_vsi {
 	u8 *rss_lut_user;	/* User configured lookup table entries */
 	u8 rss_lut_type;	/* used to configure Get/Set RSS LUT AQ call */
 
-#ifdef CONFIG_RFS_ACCEL
+#if IS_ENABLED(CONFIG_NET_DEVLINK)
+	/* devlink port data */
+	struct devlink_port devlink_port;
+	bool devlink_port_registered;
+#endif /* CONFIG_NET_DEVLINK */
+
 	/* aRFS members only allocated for the PF VSI */
 #define ICE_MAX_RFS_FILTERS	0xFFFF
-#define ICE_MAX_ARFS_LIST	BIT(10) /* 1024 entries */
+#define ICE_MAX_ARFS_LIST	1024
 #define ICE_ARFS_LST_MASK	(ICE_MAX_ARFS_LIST - 1)
 	struct hlist_head *arfs_fltr_list;
 	struct ice_arfs_active_fltr_cntrs *arfs_fltr_cntrs;
 	spinlock_t arfs_lock;	/* protects aRFS hash table and filter state */
 	atomic_t *arfs_last_fltr_id;
-#endif /* CONFIG_RFS_ACCEL */
 
 	u16 max_frame;
 	u16 rx_buf_len;
@@ -587,7 +652,7 @@ struct ice_vsi {
 	u8 irqs_ready:1;
 	u8 current_isup:1;		 /* Sync 'link up' logging */
 	u8 stat_offsets_loaded:1;
-	u8 vlan_ena:1;
+	struct ice_vsi_vlan_ops vlan_ops;
 	u16 num_vlan;
 
 
@@ -613,15 +678,26 @@ struct ice_vsi {
 	u8 xdp_mapping_mode;		 /* ICE_MAP_MODE_[CONTIG|SCATTER] */
 #endif /* HAVE_XDP_SUPPORT */
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
+#ifndef HAVE_AF_XDP_NETDEV_UMEM
 	struct xdp_umem **xsk_umems;
 	u16 num_xsk_umems_used;
 	u16 num_xsk_umems;
+#endif /* !HAVE_AF_XDP_NETDEV_UMEM */
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
 
 #ifdef HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO
 	struct tc_mqprio_qopt_offload mqprio_qopt;/* queue parameters */
 	struct ice_tcf_qreg_cfg tcf_qreg_cfg;
 #endif /* HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO */
+	DECLARE_BITMAP(ptp_tx_idx, INDEX_PER_QUAD);
+	struct sk_buff *ptp_tx_skb[INDEX_PER_QUAD];
+	struct hwtstamp_config tstamp_config;
+	struct ptp_clock_info ptp_caps;
+	struct ptp_clock *ptp_clock;
+	u32 tx_hwtstamp_skipped;
+	u8 ptp_tx:1;
+	enum ice_time_ref_freq time_ref_freq;
+	enum ice_mstr_tmr_mode mstr_tmr_mode;
 
 	/* Channel Specific Fields */
 	struct ice_vsi *tc_map_vsi[ICE_CHNL_MAX_TC];
@@ -640,7 +716,6 @@ struct ice_vsi {
 	u8 vf_adq_tc;	/* traffic class number for VF ADQ VSI */
 	/* track various feature bits for channel VSI */
 	DECLARE_BITMAP(features, ICE_CHNL_FEATURE_NBITS);
-#ifdef ADQ_PERF
 #define ICE_TBL_FULL_TIMES             5
 	/* how many times transitioned into inline flow-director from RSS */
 	u64 cnt_inline_fd_transition;
@@ -656,7 +731,6 @@ struct ice_vsi {
 	 */
 	atomic_t inline_fd_active_cnt;
 	DECLARE_BITMAP(adv_state, ICE_ADVANCED_STATE_LAST);
-#endif /* ADQ_PERF */
 
 	/* this keeps tracks of all enabled TC with and without DCB
 	 * and inclusive of ADQ, vsi->mqprio_opt keeps track of queue
@@ -670,14 +744,20 @@ struct ice_vsi {
 	u16 old_ena_tc;
 
 	struct ice_channel *ch;
+
+	/* setup back reference, to which aggregator node this VSI
+	 * corresponds to
+	 */
+	struct ice_agg_node *agg_node;
+	u16 *global_lut_id;
 } ____cacheline_internodealigned_in_smp;
 
-#ifdef ADQ_PERF
 enum ice_chnl_vector_state {
 	ICE_CHNL_VECTOR_IN_BP,
 	ICE_CHNL_VECTOR_PREV_IN_BP,
 	ICE_CHNL_VECTOR_ONCE_IN_BP,
 	ICE_CHNL_VECTOR_PREV_DATA_PKT_RECV,
+	ICE_CHNL_VECTOR_WD_EQUALS_BP,
 	ICE_CHNL_VECTOR_NBITS, /* This must be last */
 };
 
@@ -719,9 +799,17 @@ struct ice_q_vector_ch_stats {
 	u64 num_no_sw_intr_opt_off;
 	/* tracking, how many times WB_ON_ITR is set */
 	u64 num_wb_on_itr_set;
+
+	u64 pkt_bp_stop_napi_budget;
+	u64 pkt_bp_stop_bp_budget;
+
+	u64 bp_wd_equals_budget64;
+	u64 bp_wd_equals_budget8;
+
+	u64 keep_state_bp_budget64;
+	u64 keep_state_bp_budget8;
 };
 #endif /* ADQ_PERF_COUNTERS */
-#endif /* ADQ_PERF */
 
 /* struct that defines an interrupt vector */
 struct ice_q_vector {
@@ -748,7 +836,6 @@ struct ice_q_vector {
 	struct ice_channel *ch;
 
 	char name[ICE_INT_NAME_STR_LEN];
-#ifdef ADQ_PERF
 	/* This tracks current state of vector, BUSY_POLL or INTR */
 #define ICE_CHNL_IN_BP			BIT(ICE_CHNL_VECTOR_IN_BP)
 	/* This tracks prev state of vector, BUSY_POLL or INTR */
@@ -762,6 +849,12 @@ struct ice_q_vector {
 	 * on per channel enabled vector or not
 	 */
 #define ICE_CHNL_PREV_DATA_PKT_RECV	BIT(ICE_CHNL_VECTOR_PREV_DATA_PKT_RECV)
+	/* tracks if number of Rx packets processed is equal to budget or not.
+	 * It is set from napi_poll and used from ice_refresh_bp_state
+	 * to determine if internal state of vector to be kept in BUSY_POLL
+	 * or not
+	 */
+#define ICE_CHNL_WD_EQUALS_BP		BIT(ICE_CHNL_VECTOR_WD_EQUALS_BP)
 	/* it is used to keep track of various states as defined earlier
 	 * and those states are used during ADQ performance optimization
 	 */
@@ -769,14 +862,42 @@ struct ice_q_vector {
 	/* Used in logic to determine if SW inter is needed or not.
 	 * This is used only for channel enabled vector
 	 */
-	u64 jiffies;
+	u64 jiffy;
 	/* Primarily used in decision making w.r.t using inline flow-director */
 	atomic_t inline_fd_cnt;
 
+	/* This is applicable only for ADQ enabled vectors and used to avoid
+	 * situation of OS triggering ksoftirqd.
+	 *
+	 * Usually busy_poll_stop is followed by napi_schedule:napi_poll if
+	 * driver returned "budget" as part of processing packets during
+	 * busy_poll_stop. As long as driver continue to return "budget",
+	 * OS keeps calling napi_schedule upto 10 times or 2msec and then
+	 * arms the ksoftrqd.
+	 *
+	 * As part of ADQ performance optimization, it is not preferable to
+	 * let ksoftirqd run when there has been enough packets processed.
+	 * To facilitate fairness to the consumer of those packets,
+	 * do not process Rx queues after 8 times.
+	 */
+	/* following variable keeps track of how many times Rx queues were
+	 * processed when napi_poll is invoked thru napi_schedule (as a result
+	 * of returning "budget" from busy_poll_stop:napi_poll) and
+	 * work_done == budget.
+	 */
+	u8 process_rx_queues;
+
+	/* following is controlled thru' priv-flag, value of
+	 * "max_limit_process_rx_queues" becomes 8 when priv-flag is set
+	 * otherwise it is set to 4 (default)
+	 */
+#define ICE_MAX_LIMIT_PROCESS_RX_PKTS_DFLT  4
+#define ICE_MAX_LIMIT_PROCESS_RX_PKTS  8
+	u8 max_limit_process_rx_queues;
+
 #ifdef ADQ_PERF_COUNTERS
 	struct ice_q_vector_ch_stats ch_stats;
 #endif /* ADQ_PERF_COUNTERS */
-#endif /* ADQ_PERF */
 } ____cacheline_internodealigned_in_smp;
 
 enum ice_pf_flags {
@@ -789,10 +910,11 @@ enum ice_pf_flags {
 	ICE_FLAG_RSS_ENA,
 	ICE_FLAG_SRIOV_ENA,
 	ICE_FLAG_SRIOV_CAPABLE,
-	ICE_FLAG_SRIOV_CHNL_FLTR_AS_UDP,
 	ICE_FLAG_DCB_CAPABLE,
 	ICE_FLAG_DCB_ENA,
 	ICE_FLAG_FD_ENA,
+	ICE_FLAG_PTP_ENA,		/* NVM PTP support */
+	ICE_FLAG_PTP,			/* PTP successfully initialized */
 	ICE_FLAG_PEER_ENA,
 	ICE_FLAG_ADV_FEATURES,
 #ifdef NETIF_F_HW_TC
@@ -800,23 +922,22 @@ enum ice_pf_flags {
 	ICE_FLAG_CLS_FLOWER,		/* support cls flower filters */
 #endif /* NETIF_F_HW_TC */
 	ICE_FLAG_LINK_DOWN_ON_CLOSE_ENA,
+	ICE_FLAG_TOTAL_PORT_SHUTDOWN_ENA,
 	ICE_FLAG_NO_MEDIA,
 #ifndef ETHTOOL_GFECPARAM
 	ICE_FLAG_RS_FEC,
 	ICE_FLAG_BASE_R_FEC,
 #endif /* !ETHTOOL_GFECPARAM */
 	ICE_FLAG_FW_LLDP_AGENT,
-	ICE_FLAG_EXTERNAL_PHY,
-#ifdef ADQ_PERF
 	ICE_FLAG_CHNL_INLINE_FD_ENA,
-#endif /* ADQ_PERF */
-#ifdef ADQ_PERF
 	ICE_FLAG_CHNL_PKT_INSPECT_OPT_ENA,
-#endif /* ADQ_PERF */
+	ICE_FLAG_CHNL_PKT_CLEAN_BP_STOP_ENA,
+	ICE_FLAG_CHNL_PKT_CLEAN_BP_STOP_CFG,
 	ICE_FLAG_ETHTOOL_CTXT,		/* set when ethtool holds RTNL lock */
 	ICE_FLAG_LEGACY_RX,
 	ICE_FLAG_VF_TRUE_PROMISC_ENA,
 	ICE_FLAG_MDD_AUTO_RESET_VF,
+	ICE_FLAG_LINK_LENIENT_MODE_ENA,
 	ICE_PF_FLAGS_NBITS		/* must be last */
 };
 
@@ -832,9 +953,53 @@ struct ice_macvlan {
 #endif /* HAVE_NETDEV_SB_DEV */
 
 
+enum ice_tnl_state {
+	ICE_TNL_SET_TO_ADD,
+	ICE_TNL_ACTIVE,
+	ICE_TNL_SET_TO_DEL,
+	ICE_TNL_LAST = 0xFF, /* must be last */
+};
+
+struct ice_tnl_entry {
+	enum ice_tunnel_type type;
+	u16 port;
+#define ICE_TNL_STATE_TO_ADD	BIT(ICE_TNL_SET_TO_ADD)
+#define ICE_TNL_STATE_ACTIVE	BIT(ICE_TNL_ACTIVE)
+#define ICE_TNL_STATE_TO_DEL	BIT(ICE_TNL_SET_TO_DEL)
+	u8 state;
+	u8 ref_cnt;
+	struct list_head node;
+};
+
+
+struct ice_agg_node {
+	u32 agg_id;
+#define ICE_MAX_VSIS_IN_AGG_NODE	64
+	u32 num_vsis;
+	u8 valid;
+};
+
+enum ice_flash_update_preservation {
+	/* Preserve all settings and fields */
+	ICE_FLASH_UPDATE_PRESERVE_ALL = 0,
+	/* Preserve limited fields, such as VPD, PCI serial ID, MACs, etc */
+	ICE_FLASH_UPDATE_PRESERVE_LIMITED,
+	/* Return all fields to factory settings */
+	ICE_FLASH_UPDATE_PRESERVE_FACTORY_SETTINGS,
+	/* Do not perform any preservation */
+	ICE_FLASH_UPDATE_PRESERVE_NONE,
+};
+
 struct ice_pf {
 	struct pci_dev *pdev;
 
+#if IS_ENABLED(CONFIG_NET_DEVLINK)
+#ifdef HAVE_DEVLINK_REGIONS
+	struct devlink_region *nvm_region;
+	struct devlink_region *devcaps_region;
+#endif /* HAVE_DEVLINK_REGIONS */
+#endif /* CONFIG_NET_DEVLINK */
+
 	/* OS reserved IRQ details */
 	struct msix_entry *msix_entries;
 	struct ice_res_tracker *irq_tracker;
@@ -853,13 +1018,14 @@ struct ice_pf {
 #endif /* CONFIG_DEBUG_FS */
 	/* Virtchnl/SR-IOV config info */
 	struct ice_vf *vf;
-	int num_alloc_vfs;		/* actual number of VFs allocated */
+	u16 num_alloc_vfs;	/* actual number of VFs allocated */
 	u16 num_vfs_supported;		/* num VFs supported for this PF */
-	u16 num_vf_qps;			/* num queue pairs per VF */
-	u16 num_vf_msix;		/* num vectors per VF */
+	u16 num_qps_per_vf;
+	u16 num_msix_per_vf;
 	/* used to ratelimit the MDD event logging */
 	unsigned long last_printed_mdd_jiffies;
-	DECLARE_BITMAP(state, __ICE_STATE_NBITS);
+	DECLARE_BITMAP(malvfs, ICE_MAX_VF_COUNT);
+	DECLARE_BITMAP(state, ICE_STATE_NBITS);
 	DECLARE_BITMAP(flags, ICE_PF_FLAGS_NBITS);
 	unsigned long *avail_txqs;	/* bitmap to track PF Tx queue usage */
 	unsigned long *avail_rxqs;	/* bitmap to track PF Rx queue usage */
@@ -871,9 +1037,16 @@ struct ice_pf {
 	struct mutex sw_mutex;		/* lock for protecting VSI alloc flow */
 	struct mutex tc_mutex;		/* lock to protect TC changes */
 	u32 msg_enable;
-	/* Total number of MSIX vectors reserved for base driver */
-	u32 num_rdma_msix;
-	u32 rdma_base_vector;
+	u64 cached_systime;
+	u8 ptp_tx_ts_ena;
+	u8 ptp_one_pps_out_ena;
+	atomic_t ptp_phy_reset_lock;
+	struct workqueue_struct *ov_wq;
+	struct mutex ptp_ps_lock; /* protects access to PTP PHY start */
+	u8 ptp_link_up;
+	struct ice_cgu_info cgu_info;
+	u16 num_rdma_msix;	/* Total MSIX vectors for RDMA driver */
+	u16 rdma_base_vector;
 	struct ice_peer_dev *rdma_peer;
 #ifdef HAVE_NETDEV_SB_DEV
 	/* MACVLAN specific variables */
@@ -882,12 +1055,18 @@ struct ice_pf {
 	u16 num_macvlan;
 	u16 max_num_macvlan;
 #endif /* HAVE_NETDEV_SB_DEV */
+
+	/* spinlock to protect the AdminQ wait list */
+	spinlock_t aq_wait_lock;
+	struct hlist_head aq_wait_list;
+	wait_queue_head_t aq_wait_queue;
+
 	u32 hw_csum_rx_error;
-	u32 oicr_idx;		/* Other interrupt cause MSIX vector index */
-	u32 num_avail_sw_msix;	/* remaining MSIX SW vectors left unclaimed */
+	u16 oicr_idx;		/* Other interrupt cause MSIX vector index */
+	u16 num_avail_sw_msix;	/* remaining MSIX SW vectors left unclaimed */
 	u16 max_pf_txqs;	/* Total Tx queues PF wide */
 	u16 max_pf_rxqs;	/* Total Rx queues PF wide */
-	u32 num_lan_msix;	/* Total MSIX vectors for base driver */
+	u16 num_lan_msix;	/* Total MSIX vectors for base driver */
 	u16 num_lan_tx;		/* num LAN Tx queues setup */
 	u16 num_lan_rx;		/* num LAN Rx queues setup */
 	u16 next_vsi;		/* Next free slot in pf->vsi[] - 0-based! */
@@ -897,11 +1076,12 @@ struct ice_pf {
 	u16 empr_count;		/* EMP reset count */
 	u16 pfr_count;		/* PF reset count */
 
+	u8 stat_prev_loaded : 1; /* has previous stats been loaded */
+	u8 wol_ena : 1;		/* software state of WoL */
+	u32 wakeup_reason;	/* last wakeup reason */
 	struct ice_hw_port_stats stats;
 	struct ice_hw_port_stats stats_prev;
 	struct ice_hw hw;
-	u8 stat_prev_loaded:1; /* has previous stats been loaded */
-	u8 wol_ena:1;
 #ifdef ICE_ADD_PROBES
 	u64 tcp_segs;
 	u64 udp_segs;
@@ -931,10 +1111,42 @@ struct ice_pf {
 	int peer_idx;
 	u32 sw_int_count;
 #ifdef HAVE_TC_SETUP_CLSFLOWER
+	/* count of tc_flower filters specific to channel (aka where filter
+	 * action is "hw_tc <tc_num>")
+	 */
+	u16 num_dmac_chnl_fltrs;
 	struct hlist_head tc_flower_fltr_list;
-	u16 num_tc_flower_fltrs;
 #endif /* HAVE_TC_SETUP_CLSFLOWER */
 
+	struct ice_dcf dcf;
+	__le64 nvm_phy_type_lo; /* NVM PHY type low */
+	__le64 nvm_phy_type_hi; /* NVM PHY type high */
+	struct ice_link_default_override_tlv link_dflt_override;
+	u64 supported_rxdids; /* bitmap for supported RXDID */
+#ifdef HAVE_NETDEV_UPPER_INFO
+	struct ice_lag *lag; /* Link Aggregation information */
+#endif /* HAVE_NETDEV_UPPER_INFO */
+
+	/* protects accesses to tunnel list, it is grabbed
+	 * from ice_udp_tunnel_add/del and as well from service_task
+	 */
+	spinlock_t tnl_lock;
+	struct list_head tnl_list;
+
+#define ICE_INVALID_AGG_NODE_ID		0
+#define ICE_PF_AGG_NODE_ID_START	1
+#define ICE_MAX_PF_AGG_NODES		32
+	struct ice_agg_node pf_agg_node[ICE_MAX_PF_AGG_NODES];
+#ifdef HAVE_NETDEV_SB_DEV
+#define ICE_MACVLAN_AGG_NODE_ID_START	(ICE_PF_AGG_NODE_ID_START + \
+					 ICE_MAX_PF_AGG_NODES)
+#define ICE_MAX_MACVLAN_AGG_NODES	32
+	struct ice_agg_node macvlan_agg_node[ICE_MAX_MACVLAN_AGG_NODES];
+#endif
+#define ICE_VF_AGG_NODE_ID_START	65
+#define ICE_MAX_VF_AGG_NODES		32
+	struct ice_agg_node vf_agg_node[ICE_MAX_VF_AGG_NODES];
+
 #ifdef HAVE_PF_RING
 	u16 instance; /* A unique number per ice_pf instance in the system */
 	struct {
@@ -959,14 +1171,15 @@ struct ice_netdev_priv {
 	 *
 	 */
 	struct list_head tc_indr_block_priv_list;
+#ifndef HAVE_TC_FLOW_INDIR_DEV
 	struct notifier_block netdevice_nb;
+#endif
 #endif /* HAVE_TC_INDIR_BLOCK */
 };
 
 extern struct ida ice_peer_index_ida;
 
 
-#ifdef ADQ_PERF
 /**
  * ice_vector_ch_enabled
  * @qv: pointer to q_vector, can be NULL
@@ -1041,13 +1254,19 @@ static inline bool ice_vector_intr_busypoll(struct ice_q_vector *qv)
 static inline void
 ice_adq_trigger_sw_intr(struct ice_hw *hw, struct ice_q_vector *q_vector)
 {
+	struct ice_ring_container *rx_rc = &q_vector->rx;
+
 	q_vector->state_flags &= ~ICE_CHNL_ONCE_IN_BP;
 
+	/* when entering into interrupt mode, use current value of Rx ITR
+	 * hence rx_rc->itr_setting. This is needed to honor user setting
+	 * for Rx ITR
+	 */
 	wr32(hw,
 	     GLINT_DYN_CTL(q_vector->reg_idx),
-	     (ICE_ITR_NONE << GLINT_DYN_CTL_ITR_INDX_S) |
+	     (rx_rc->itr_idx << GLINT_DYN_CTL_ITR_INDX_S) |
+	     (ITR_REG_ALIGN(rx_rc->itr_setting) >> ICE_ITR_GRAN_S) |
 	     GLINT_DYN_CTL_SWINT_TRIG_M |
-	     GLINT_DYN_CTL_CLEARPBA_M |
 	     GLINT_DYN_CTL_INTENA_M);
 }
 
@@ -1074,7 +1293,7 @@ ice_sw_intr_cntr(struct ice_q_vector *q_vector, bool napi_codepath)
 /**
  * ice_force_wb - trigger force write-back by setting WB_ON_ITR bit
  * @hw: ptr to HW
- * @qv: pointer to q_vector
+ * @q_vector: pointer to q_vector
  *
  * This function is used to force write-backs by setting WB_ON_ITR bit
  * in DYN_CTLN register. WB_ON_ITR and INTENA are mutually exclusive bits.
@@ -1084,21 +1303,43 @@ ice_sw_intr_cntr(struct ice_q_vector *q_vector, bool napi_codepath)
 static inline void
 ice_force_wb(struct ice_hw *hw, struct ice_q_vector *q_vector)
 {
+	if (q_vector->num_ring_rx || q_vector->num_ring_tx) {
 #ifdef ADQ_PERF_COUNTERS
-	q_vector->ch_stats.num_wb_on_itr_set++;
-
+		q_vector->ch_stats.num_wb_on_itr_set++;
 #endif /* ADQ_PERF_COUNTERS */
-	wr32(hw,
-	     GLINT_DYN_CTL(q_vector->reg_idx),
-	     (ICE_ITR_NONE << GLINT_DYN_CTL_ITR_INDX_S) |
-	     GLINT_DYN_CTL_WB_ON_ITR_M);
+		wr32(hw, GLINT_DYN_CTL(q_vector->reg_idx),
+		     ICE_GLINT_DYN_CTL_WB_ON_ITR(0, ICE_RX_ITR));
+	}
 
 	/* needed to avoid triggering WB_ON_ITR again which typically
 	 * happens from ice_set_wb_on_itr function
 	 */
 	q_vector->itr_countdown = ICE_IN_WB_ON_ITR_MODE;
 }
-#endif /* ADQ_PERF */
+
+#ifdef HAVE_TC_SETUP_CLSFLOWER
+/**
+ * ice_is_chnl_fltr - is this a valid channel filter
+ * @f: Pointer to tc-flower filter
+ *
+ * Criteria to determine of given filter is valid channel filter
+ * or not is based on its "destination". If destination is hw_tc (aka tc_class)
+ * and it is non-zero, then it is valid channel (aka ADQ) filter
+ */
+static inline bool ice_is_chnl_fltr(struct ice_tc_flower_fltr *f)
+{
+	return !!f->action.tc_class;
+}
+
+/**
+ * ice_chnl_dmac_fltr_cnt - DMAC based CHNL filter count
+ * @pf: Pointer to PF
+ */
+static inline int ice_chnl_dmac_fltr_cnt(struct ice_pf *pf)
+{
+	return pf->num_dmac_chnl_fltrs;
+}
+#endif /* HAVE_TC_SETUP_CLSFLOWER */
 
 /**
  * ice_irq_dynamic_ena - Enable default interrupt generation settings
@@ -1121,7 +1362,7 @@ ice_irq_dynamic_ena(struct ice_hw *hw, struct ice_vsi *vsi,
 	val = GLINT_DYN_CTL_INTENA_M | GLINT_DYN_CTL_CLEARPBA_M |
 	      (itr << GLINT_DYN_CTL_ITR_INDX_S);
 	if (vsi)
-		if (test_bit(__ICE_DOWN, vsi->state))
+		if (test_bit(ICE_VSI_DOWN, vsi->state))
 			return;
 	wr32(hw, GLINT_DYN_CTL(vector), val);
 }
@@ -1152,24 +1393,33 @@ static inline void ice_set_ring_xdp(struct ice_ring *ring)
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
 /**
  * ice_xsk_umem - get XDP UMEM bound to a ring
- * @ring - ring to use
+ * @ring: ring to use
  *
  * Returns a pointer to xdp_umem structure if there is an UMEM present,
  * NULL otherwise.
  */
 static inline struct xdp_umem *ice_xsk_umem(struct ice_ring *ring)
 {
+#ifndef HAVE_AF_XDP_NETDEV_UMEM
 	struct xdp_umem **umems = ring->vsi->xsk_umems;
+#endif /* !HAVE_AF_XDP_NETDEV_UMEM */
 	u16 qid = ring->q_index;
 
 	if (ice_ring_is_xdp(ring))
 		qid -= ring->vsi->num_xdp_txq;
 
+#ifndef HAVE_AF_XDP_NETDEV_UMEM
 	if (qid >= ring->vsi->num_xsk_umems || !umems || !umems[qid] ||
 	    !ice_is_xdp_ena_vsi(ring->vsi))
 		return NULL;
 
 	return umems[qid];
+#else
+	if (!ice_is_xdp_ena_vsi(ring->vsi))
+		return NULL;
+
+	return xdp_get_umem_from_qid(ring->vsi->netdev, qid);
+#endif /* !HAVE_AF_XDP_NETDEV_UMEM */
 }
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
 
@@ -1187,6 +1437,15 @@ static inline struct ice_vsi *ice_get_main_vsi(struct ice_pf *pf)
 	return NULL;
 }
 
+/**
+ * ice_get_netdev_priv_vsi - return VSI associated with netdev priv.
+ * @np: private netdev structure
+ */
+static inline struct ice_vsi *ice_get_netdev_priv_vsi(struct ice_netdev_priv *np)
+{
+	return np->vsi;
+}
+
 /**
  * ice_get_ctrl_vsi - Get the control VSI
  * @pf: PF instance
@@ -1203,17 +1462,17 @@ static inline struct ice_vsi *ice_get_ctrl_vsi(struct ice_pf *pf)
 /**
  * ice_find_first_vsi_by_type - Find and return first VSI of a given type
  * @pf: PF to search for VSI
- * @type: VSI type we are looking for
+ * @vsi_type: VSI type we are looking for
  */
 static inline struct ice_vsi *
-ice_find_first_vsi_by_type(struct ice_pf *pf, enum ice_vsi_type type)
+ice_find_first_vsi_by_type(struct ice_pf *pf, enum ice_vsi_type vsi_type)
 {
 	int i;
 
 	ice_for_each_vsi(pf, i) {
 		struct ice_vsi *vsi = pf->vsi[i];
 
-		if (vsi && vsi->type == type)
+		if (vsi && vsi->type == vsi_type)
 			return vsi;
 	}
 
@@ -1222,9 +1481,7 @@ ice_find_first_vsi_by_type(struct ice_pf *pf, enum ice_vsi_type type)
 
 enum ice_fd_stat_idx {
 	ICE_FD_STAT_SB,
-#ifdef ADQ_PERF
 	ICE_FD_STAT_CH,
-#endif /* ADQ_PERF */
 #ifdef ICE_ADD_PROBES
 	ICE_ARFS_STAT_TCPV4,
 	ICE_ARFS_STAT_TCPV6,
@@ -1249,7 +1506,6 @@ enum ice_fd_stat_idx {
 			(ICE_FD_STAT_PF_IDX(base_idx) + ICE_ARFS_STAT_UDPV6)
 #endif /* ICE_ADD_PROBES */
 
-#ifdef ADQ_PERF
 #define ICE_FD_CH_STAT_IDX(base_idx) \
 			(ICE_FD_STAT_PF_IDX(base_idx) + ICE_FD_STAT_CH)
 
@@ -1326,15 +1582,20 @@ static inline void ice_clear_cntr(struct ice_pf *pf, u32 counter_index)
  */
 static inline bool ice_is_vsi_fd_table_full(struct ice_vsi *vsi, u32 cnt)
 {
+	u32 max_allowed_fltr_cnt;
+
 	if (!cnt)
 		return false;
 
+	if (!vsi->num_gfltr && !vsi->num_bfltr)
+		return false;
 	/* determine if 'cnt' reached max_allowed for specified VSI,
 	 * if so, return HW table full for that specific VSI
 	 */
-	return (cnt >= (vsi->num_gfltr + vsi->num_bfltr - 1)) ? true : false;
+	max_allowed_fltr_cnt = vsi->num_gfltr + vsi->num_bfltr - 1;
+
+	return cnt >= max_allowed_fltr_cnt;
 }
-#endif /* ADQ_PERF */
 
 #ifdef NETIF_F_HW_TC
 /**
@@ -1361,7 +1622,6 @@ static inline bool ice_is_adq_active(struct ice_pf *pf)
 	return false;
 }
 #endif /* NETIF_F_HW_TC */
-#ifdef ADQ_PERF
 /**
  * ice_vsi_pkt_inspect_opt_ena - packet inspection based optimization is ON/OFF
  * @vsi: pointer to VSI
@@ -1377,33 +1637,47 @@ static inline bool ice_vsi_pkt_inspect_opt_ena(struct ice_vsi *vsi)
 {
 	return !!test_bit(ICE_CHNL_FEATURE_PKT_INSPECT_OPT_ENA, vsi->features);
 }
-#endif /* ADQ_PERF */
+
+/**
+ * ice_vsi_pkt_process_bp_stop_ena - packet process ON/OFF from bp stop
+ * @vsi: pointer to VSI
+ *
+ * This function returns true if VSI is enabled for optimization to allow
+ * Tx/Rx cleanup from busy_poll_stop code path. There is an associated
+ * priv flag to control this feature and applicable only for channel (aka ADQ)
+ * specific vectors
+ */
+static inline bool ice_vsi_pkt_process_bp_stop_ena(struct ice_vsi *vsi)
+{
+	return !!test_bit(ICE_CHNL_FEATURE_PKT_CLEAN_BP_STOP_ENA,
+			  vsi->features);
+}
 
 static inline bool ice_active_vmdqs(struct ice_pf *pf)
 {
 	return !!ice_find_first_vsi_by_type(pf, ICE_VSI_VMDQ2);
 }
 
+#ifdef HAVE_NETDEV_SB_DEV
+static inline bool ice_is_offloaded_macvlan_ena(struct ice_pf *pf)
+{
+	return test_bit(ICE_FLAG_MACVLAN_ENA, pf->flags);
+}
+#endif /* HAVE_NETDEV_SB_DEV */
+
 #ifdef CONFIG_DEBUG_FS
 void ice_debugfs_pf_init(struct ice_pf *pf);
 void ice_debugfs_pf_exit(struct ice_pf *pf);
 void ice_debugfs_init(void);
 void ice_debugfs_exit(void);
 #else
-static inline void ice_debugfs_pf_init(struct ice_pf *pf) {}
-static inline void ice_debugfs_pf_exit(struct ice_pf *pf) {}
-static inline void ice_debugfs_init(void) {}
-static inline void ice_debugfs_exit(void) {}
+#define ice_debugfs_pf_init(pf) do {} while (0)
+#define ice_debugfs_pf_exit(pf) do {} while (0)
+#define ice_debugfs_init() do {} while (0)
+#define ice_debugfs_exit() do {} while (0)
 #endif /* CONFIG_DEBUG_FS */
 
-#ifdef HAVE_NETDEV_SB_DEV
 bool netif_is_ice(struct net_device *dev);
-static inline bool ice_is_offloaded_macvlan_ena(struct ice_pf *pf)
-{
-	return test_bit(ICE_FLAG_MACVLAN_ENA, pf->flags);
-}
-#endif /* HAVE_NETDEV_SB_DEV */
-
 int ice_vsi_setup_tx_rings(struct ice_vsi *vsi);
 int ice_vsi_setup_rx_rings(struct ice_vsi *vsi);
 int ice_vsi_open_ctrl(struct ice_vsi *vsi);
@@ -1437,8 +1711,10 @@ ice_xdp_xmit(struct net_device *dev, int n, struct xdp_frame **frames,
 int ice_xdp_xmit(struct net_device *dev, struct xdp_buff *xdp);
 #endif /* HAVE_XDP_FRAME_STRUCT */
 #endif /* HAVE_XDP_SUPPORT */
-int ice_set_rss(struct ice_vsi *vsi, u8 *seed, u8 *lut, u16 lut_size);
-int ice_get_rss(struct ice_vsi *vsi, u8 *seed, u8 *lut, u16 lut_size);
+int ice_set_rss_lut(struct ice_vsi *vsi, u8 *lut, u16 lut_size);
+int ice_get_rss_lut(struct ice_vsi *vsi, u8 *lut, u16 lut_size);
+int ice_set_rss_key(struct ice_vsi *vsi, u8 *seed);
+int ice_get_rss_key(struct ice_vsi *vsi, u8 *seed);
 void ice_fill_rss_lut(u8 *lut, u16 rss_table_size, u16 rss_size);
 int ice_schedule_reset(struct ice_pf *pf, enum ice_reset_req reset);
 void ice_print_link_msg(struct ice_vsi *vsi, bool isup);
@@ -1464,21 +1740,69 @@ ice_for_each_peer(struct ice_pf *pf, void *data,
 #define ice_peer_refresh_msix(pf) do { } while (0)
 #endif /* CONFIG_PM */
 #endif /* !CONFIG_MFD_CORE */
+#ifdef HAVE_NETDEV_UPPER_INFO
+/**
+ * ice_set_sriov_cap - enable SRIOV in PF flags
+ * @pf: PF struct
+ */
+static inline void ice_set_sriov_cap(struct ice_pf *pf)
+{
+	if (pf->hw.func_caps.common_cap.sr_iov_1_1)
+		set_bit(ICE_FLAG_SRIOV_CAPABLE, pf->flags);
+}
 
+/**
+ * ice_clear_sriov_cap - disable SRIOV in PF flags
+ * @pf: PF struct
+ */
+static inline void ice_clear_sriov_cap(struct ice_pf *pf)
+{
+	clear_bit(ICE_FLAG_SRIOV_CAPABLE, pf->flags);
+}
+
+/**
+ * ice_set_rdma_cap - enable RDMA in PF flags
+ * @pf: PF struct
+ */
+static inline void ice_set_rdma_cap(struct ice_pf *pf)
+{
+	if (pf->hw.func_caps.common_cap.iwarp)
+		set_bit(ICE_FLAG_IWARP_ENA, pf->flags);
+}
+
+/**
+ * ice_clear_rdma_cap - disable RDMA in PF flags
+ * @pf: PF struct
+ */
+static inline void ice_clear_rdma_cap(struct ice_pf *pf)
+{
+	clear_bit(ICE_FLAG_IWARP_ENA, pf->flags);
+}
+
+#endif /* HAVE_NETDEV_UPPER_INFO */
 const char *ice_stat_str(enum ice_status stat_err);
 const char *ice_aq_str(enum ice_aq_err aq_err);
-bool ice_is_wol_ena(struct ice_pf *pf);
-int
-ice_prgm_fdir_fltr(struct ice_vsi *vsi, struct ice_fltr_desc *fdir_desc,
-		   u8 *raw_packet);
+bool ice_is_wol_supported(struct ice_pf *pf);
+int ice_aq_wait_for_event(struct ice_pf *pf, u16 opcode, unsigned long timeout,
+			  struct ice_rq_event_info *event);
 int
 ice_fdir_write_fltr(struct ice_pf *pf, struct ice_fdir_fltr *input, bool add,
 		    bool is_tun);
-void ice_vsi_drain_rxq(struct ice_vsi *vsi, int idx);
 void ice_vsi_manage_fdir(struct ice_vsi *vsi, bool ena);
-int ice_add_fdir_ethtool(struct ice_vsi *vsi, struct ethtool_rxnfc *cmd);
-int ice_del_fdir_ethtool(struct ice_vsi *vsi, struct ethtool_rxnfc *cmd);
+int ice_add_ntuple_ethtool(struct ice_vsi *vsi, struct ethtool_rxnfc *cmd);
+int ice_del_ntuple_ethtool(struct ice_vsi *vsi, struct ethtool_rxnfc *cmd);
 int ice_get_ethtool_fdir_entry(struct ice_hw *hw, struct ethtool_rxnfc *cmd);
+u32 ice_ntuple_get_max_fltr_cnt(struct ice_hw *hw);
+int
+ice_ntuple_set_input_set(struct ice_vsi *vsi, enum ice_block blk,
+			 struct ethtool_rx_flow_spec *fsp,
+			 struct ice_fdir_fltr *input);
+int
+ice_ntuple_l4_proto_to_port(enum ice_flow_seg_hdr l4_proto,
+			    enum ice_flow_field *src_port,
+			    enum ice_flow_field *dst_port);
+int ice_ntuple_check_ip4_seg(struct ethtool_tcpip4_spec *tcp_ip4_spec);
+int ice_ntuple_check_ip4_usr_seg(struct ethtool_usrip4_spec *usr_ip4_spec);
 int
 ice_get_fdir_fltr_ids(struct ice_hw *hw, struct ethtool_rxnfc *cmd,
 		      u32 *rule_locs);
@@ -1486,11 +1810,19 @@ void ice_fdir_rem_adq_chnl(struct ice_hw *hw, u16 vsi_idx);
 void ice_fdir_release_flows(struct ice_hw *hw);
 void ice_fdir_replay_flows(struct ice_hw *hw);
 void ice_fdir_replay_fltrs(struct ice_pf *pf);
-int ice_create_fdir_rule(struct ice_pf *pf, enum ice_fltr_ptype flow);
+int ice_fdir_create_dflt_rules(struct ice_pf *pf);
+enum ice_fltr_ptype ice_ethtool_flow_to_fltr(int eth);
+int
+ice_ntuple_update_list_entry(struct ice_pf *pf, struct ice_fdir_fltr *input,
+			     int fltr_idx);
+void ice_update_ring_dest_vsi(struct ice_vsi *vsi, u16 *dest_vsi, u32 *ring);
 int ice_open(struct net_device *netdev);
+int ice_open_internal(struct net_device *netdev);
 int ice_stop(struct net_device *netdev);
 void ice_service_task_schedule(struct ice_pf *pf);
-
+int
+ice_acl_add_rule_ethtool(struct ice_vsi *vsi, struct ethtool_rxnfc *cmd);
+int ice_init_acl(struct ice_pf *pf);
 #ifdef HAVE_TC_SETUP_CLSFLOWER
 int
 ice_add_tc_flower_adv_fltr(struct ice_vsi *vsi,
