@@ -12,6 +12,7 @@
 #define ICE_ACL_TBL_TCAM_ENTRY_IDX(e) ((e) % ICE_AQC_ACL_TCAM_DEPTH)
 
 #define ICE_ACL_SCEN_ENTRY_INVAL 0xFFFF
+
 /**
  * ice_acl_init_entry
  * @scen: pointer to the scenario struct
@@ -20,41 +21,43 @@
  */
 static void ice_acl_init_entry(struct ice_acl_scen *scen)
 {
-	/*
-	 * low priority: start from the highest index, 25% of total entries
+	/* low priority: start from the highest index, 25% of total entries
 	 * normal priority: start from the highest index, 50% of total entries
 	 * high priority: start from the lowest index, 25% of total entries
 	 */
-	scen->first_idx[ICE_LOW] = scen->num_entry - 1;
-	scen->first_idx[ICE_NORMAL] = scen->num_entry - scen->num_entry / 4 - 1;
-	scen->first_idx[ICE_HIGH] = 0;
+	scen->first_idx[ICE_ACL_PRIO_LOW] = scen->num_entry - 1;
+	scen->first_idx[ICE_ACL_PRIO_NORMAL] = scen->num_entry -
+		scen->num_entry / 4 - 1;
+	scen->first_idx[ICE_ACL_PRIO_HIGH] = 0;
 
-	scen->last_idx[ICE_LOW] = scen->num_entry - scen->num_entry / 4;
-	scen->last_idx[ICE_NORMAL] = scen->num_entry / 4;
-	scen->last_idx[ICE_HIGH] = scen->num_entry / 4 - 1;
+	scen->last_idx[ICE_ACL_PRIO_LOW] = scen->num_entry -
+		scen->num_entry / 4;
+	scen->last_idx[ICE_ACL_PRIO_NORMAL] = scen->num_entry / 4;
+	scen->last_idx[ICE_ACL_PRIO_HIGH] = scen->num_entry / 4 - 1;
 }
 
 /**
  * ice_acl_scen_assign_entry_idx
  * @scen: pointer to the scenario struct
- * @prior: the priority of the flow entry being allocated
+ * @prio: the priority of the flow entry being allocated
  *
  * To find the index of an available entry in scenario
  *
  * Returns ICE_ACL_SCEN_ENTRY_INVAL if fails
  * Returns index on success
  */
-static u16 ice_acl_scen_assign_entry_idx(struct ice_acl_scen *scen,
-					 enum ice_acl_entry_prior prior)
+static u16
+ice_acl_scen_assign_entry_idx(struct ice_acl_scen *scen,
+			      enum ice_acl_entry_prio prio)
 {
 	u16 first_idx, last_idx, i;
 	s8 step;
 
-	if (prior >= ICE_MAX_PRIOR)
+	if (prio >= ICE_ACL_MAX_PRIO)
 		return ICE_ACL_SCEN_ENTRY_INVAL;
 
-	first_idx = scen->first_idx[prior];
-	last_idx = scen->last_idx[prior];
+	first_idx = scen->first_idx[prio];
+	last_idx = scen->last_idx[prio];
 	step = first_idx <= last_idx ? 1 : -1;
 
 	for (i = first_idx; i != last_idx + step; i += step)
@@ -144,14 +147,12 @@ static enum ice_status ice_acl_init_tbl(struct ice_hw *hw)
 	struct ice_aqc_acl_data buf;
 	enum ice_status status = 0;
 	struct ice_acl_tbl *tbl;
-	u8 tcam_idx;
-	u16 idx, i;
+	u8 tcam_idx, i;
+	u16 idx;
 
 	tbl = hw->acl_tbl;
-	if (!tbl) {
-		status = ICE_ERR_CFG;
-		return status;
-	}
+	if (!tbl)
+		return ICE_ERR_CFG;
 
 	memset(&buf, 0, sizeof(buf));
 	memset(&act_buf, 0, sizeof(act_buf));
@@ -168,26 +169,20 @@ static enum ice_status ice_acl_init_tbl(struct ice_hw *hw)
 		if (status)
 			return status;
 
-		if (++idx == ICE_AQC_ACL_TCAM_DEPTH) {
+		if (++idx > tbl->last_entry) {
 			tcam_idx++;
-			idx = 0;
+			idx = tbl->first_entry;
 		}
 	}
 
 	for (i = 0; i < ICE_AQC_MAX_ACTION_MEMORIES; i++) {
-		u8 member_of_tcam = tbl->act_mems[i].member_of_tcam;
-		u16 end = ICE_AQC_MAX_ACTION_ENTRIES - 1;
-		u16 act_entry_idx;
-		u16 start = 0;
+		u16 act_entry_idx, start, end;
 
 		if (tbl->act_mems[i].act_mem == ICE_ACL_ACT_PAIR_MEM_INVAL)
 			continue;
 
-		if (member_of_tcam == tbl->first_tcam)
-			start = tbl->first_entry;
-
-		if (member_of_tcam == tbl->last_tcam)
-			end = tbl->last_entry;
+		start = tbl->first_entry;
+		end = tbl->last_entry;
 
 		for (act_entry_idx = start; act_entry_idx <= end;
 		     act_entry_idx++) {
@@ -204,7 +199,7 @@ static enum ice_status ice_acl_init_tbl(struct ice_hw *hw)
 
 /**
  * ice_acl_assign_act_mems_to_tcam
- * @tbl: pointer to acl table structure
+ * @tbl: pointer to ACL table structure
  * @cur_tcam: Index of current TCAM. Value = 0 to (ICE_AQC_ACL_SLICES - 1)
  * @cur_mem_idx: Index of current action memory bank. Value = 0 to
  *		 (ICE_AQC_MAX_ACTION_MEMORIES - 1)
@@ -214,7 +209,7 @@ static enum ice_status ice_acl_init_tbl(struct ice_hw *hw)
  * "curr_tcam" TCAM.
  */
 static void
-ice_acl_assign_act_mems_to_tcam(struct ice_acl_tbl *tbl, u16 cur_tcam,
+ice_acl_assign_act_mems_to_tcam(struct ice_acl_tbl *tbl, u8 cur_tcam,
 				u8 *cur_mem_idx, u8 num_mem)
 {
 	u8 mem_cnt;
@@ -235,7 +230,7 @@ ice_acl_assign_act_mems_to_tcam(struct ice_acl_tbl *tbl, u16 cur_tcam,
 
 /**
  * ice_acl_divide_act_mems_to_tcams
- * @tbl: pointer to acl table structure
+ * @tbl: pointer to ACL table structure
  *
  * Figure out how to divide given action memory banks to given TCAMs. This
  * division is for SW book keeping. In the time when scenario is created,
@@ -257,7 +252,7 @@ ice_acl_assign_act_mems_to_tcam(struct ice_acl_tbl *tbl, u16 cur_tcam,
 static void ice_acl_divide_act_mems_to_tcams(struct ice_acl_tbl *tbl)
 {
 	u16 num_cscd, stack_level, stack_idx, min_act_mem;
-	u16 tcam_idx = tbl->first_tcam;
+	u8 tcam_idx = tbl->first_tcam;
 	u16 max_idx_to_get_extra;
 	u8 mem_idx = 0;
 
@@ -317,6 +312,9 @@ ice_acl_create_tbl(struct ice_hw *hw, struct ice_acl_tbl_params *params)
 	struct ice_acl_tbl *tbl;
 	enum ice_status status;
 
+	if (hw->dcf_acl_enabled)
+		return ICE_ERR_IN_USE;
+
 	if (hw->acl_tbl)
 		return ICE_ERR_ALREADY_EXISTS;
 
@@ -324,13 +322,17 @@ ice_acl_create_tbl(struct ice_hw *hw, struct ice_acl_tbl_params *params)
 		return ICE_ERR_PARAM;
 
 	/* round up the width to the next TCAM width boundary. */
-	width = roundup(params->width, ICE_AQC_ACL_KEY_WIDTH_BYTES);
+	width = roundup(params->width, (u16)ICE_AQC_ACL_KEY_WIDTH_BYTES);
 	/* depth should be provided in chunk (64 entry) increments */
 	depth = ALIGN(params->depth, ICE_ACL_ENTRY_ALLOC_UNIT);
 
-	if (params->entry_act_pairs < width / ICE_AQC_ACL_KEY_WIDTH_BYTES)
+	if (params->entry_act_pairs < width / ICE_AQC_ACL_KEY_WIDTH_BYTES) {
 		params->entry_act_pairs = width / ICE_AQC_ACL_KEY_WIDTH_BYTES;
 
+		if (params->entry_act_pairs > ICE_AQC_TBL_MAX_ACTION_PAIRS)
+			params->entry_act_pairs = ICE_AQC_TBL_MAX_ACTION_PAIRS;
+	}
+
 	/* Validate that width*depth will not exceed the TCAM limit */
 	if ((DIV_ROUND_UP(depth, ICE_AQC_ACL_TCAM_DEPTH) *
 	     (width / ICE_AQC_ACL_KEY_WIDTH_BYTES)) > ICE_AQC_ACL_SLICES)
@@ -351,18 +353,15 @@ ice_acl_create_tbl(struct ice_hw *hw, struct ice_acl_tbl_params *params)
 				cpu_to_le16(params->dep_tbls[i]);
 	}
 
-	/* call the aq command to create the ACL table with these values */
+	/* call the AQ command to create the ACL table with these values */
 	status = ice_aq_alloc_acl_tbl(hw, &tbl_alloc, NULL);
-
 	if (status) {
 		if (le16_to_cpu(tbl_alloc.buf.resp_buf.alloc_id) <
 		    ICE_AQC_ALLOC_ID_LESS_THAN_4K)
-			ice_debug(hw, ICE_DBG_ACL,
-				  "Alloc ACL table failed. Unavailable resource.\n");
+			ice_debug(hw, ICE_DBG_ACL, "Alloc ACL table failed. Unavailable resource.\n");
 		else
-			ice_debug(hw, ICE_DBG_ACL,
-				  "AQ allocation of ACL failed with error. status: %d\n",
-				   status);
+			ice_debug(hw, ICE_DBG_ACL, "AQ allocation of ACL failed with error. status: %d\n",
+				  status);
 		return status;
 	}
 
@@ -402,8 +401,7 @@ ice_acl_create_tbl(struct ice_hw *hw, struct ice_acl_tbl_params *params)
 	if (status) {
 		devm_kfree(ice_hw_to_dev(hw), tbl);
 		hw->acl_tbl = NULL;
-		ice_debug(hw, ICE_DBG_ACL,
-			  "Initialization of TCAM entries failed. status: %d\n",
+		ice_debug(hw, ICE_DBG_ACL, "Initialization of TCAM entries failed. status: %d\n",
 			  status);
 		goto out;
 	}
@@ -414,8 +412,7 @@ ice_acl_create_tbl(struct ice_hw *hw, struct ice_acl_tbl_params *params)
 		(tbl->last_entry / ICE_ACL_ENTRY_ALLOC_UNIT);
 
 	/* Indicate available entries in the table */
-	for (i = first_e; i <= last_e; i++)
-		set_bit(i, tbl->avail);
+	bitmap_set(tbl->avail, first_e, last_e - first_e + 1);
 
 	INIT_LIST_HEAD(&tbl->scens);
 out:
@@ -530,7 +527,7 @@ ice_acl_alloc_partition(struct ice_hw *hw, struct ice_acl_scen *req)
 			break;
 		}
 
-		row = (dir > 0) ? (row + width) : (row - width);
+		row = dir > 0 ? row + width : row - width;
 		if (row > hw->acl_tbl->last_tcam ||
 		    row < hw->acl_tbl->first_tcam) {
 			/* All rows have been checked. Increment 'off' that
@@ -589,7 +586,8 @@ ice_acl_fill_tcam_select(struct ice_aqc_acl_scen *scen_buf,
 	 * cascade row selects indices starting with 5-n
 	 */
 	for (j = 0; j < ICE_AQC_ACL_KEY_WIDTH_BYTES; j++) {
-		u8 val = ICE_AQC_ACL_BYTE_SEL_BASE + idx;
+		/* PKT DIR uses the 1st location of Byte Selection Base: + 1 */
+		u8 val = ICE_AQC_ACL_BYTE_SEL_BASE + 1 + idx;
 
 		if (tcam_idx_in_cascade == cascade_cnt - 1) {
 			if (j == ICE_ACL_SCEN_RNG_CHK_IDX_IN_TCAM)
@@ -630,8 +628,8 @@ ice_acl_set_scen_chnk_msk(struct ice_aqc_acl_scen *scen_buf,
 
 	/* Determine the starting TCAM index and offset of the start entry */
 	tcam_idx = ICE_ACL_TBL_TCAM_IDX(scen->start);
-	chnk_offst = (u8)(scen->start % ICE_AQC_ACL_TCAM_DEPTH) /
-		ICE_ACL_ENTRY_ALLOC_UNIT;
+	chnk_offst = (u8)((scen->start % ICE_AQC_ACL_TCAM_DEPTH) /
+			  ICE_ACL_ENTRY_ALLOC_UNIT);
 
 	/* Entries are allocated and tracked in multiple of 64's */
 	units = scen->num_entry / ICE_ACL_ENTRY_ALLOC_UNIT;
@@ -658,7 +656,7 @@ ice_acl_set_scen_chnk_msk(struct ice_aqc_acl_scen *scen_buf,
 
 /**
  * ice_acl_assign_act_mem_for_scen
- * @tbl: pointer to acl table structure
+ * @tbl: pointer to ACL table structure
  * @scen: pointer to the scenario struct
  * @scen_buf: pointer to the available space for the scenario
  * @current_tcam_idx: theoretical index of the TCAM that we associated those
@@ -670,8 +668,7 @@ static void
 ice_acl_assign_act_mem_for_scen(struct ice_acl_tbl *tbl,
 				struct ice_acl_scen *scen,
 				struct ice_aqc_acl_scen *scen_buf,
-				u16 current_tcam_idx,
-				u16 target_tcam_idx)
+				u8 current_tcam_idx, u8 target_tcam_idx)
 {
 	u8 i;
 
@@ -745,11 +742,13 @@ enum ice_status
 ice_acl_create_scen(struct ice_hw *hw, u16 match_width, u16 num_entries,
 		    u16 *scen_id)
 {
+	u8 cascade_cnt, first_tcam, last_tcam, i, k;
 	struct ice_aqc_acl_scen scen_buf;
-	u16 first_tcam, last_tcam, i, k;
 	struct ice_acl_scen *scen;
 	enum ice_status status;
-	u16 cascade_cnt;
+
+	if (hw->dcf_acl_enabled)
+		return ICE_ERR_IN_USE;
 
 	if (!hw->acl_tbl)
 		return ICE_ERR_DOES_NOT_EXIST;
@@ -758,16 +757,14 @@ ice_acl_create_scen(struct ice_hw *hw, u16 match_width, u16 num_entries,
 	if (!scen)
 		return ICE_ERR_NO_MEMORY;
 
-	scen->start = 0;
+	scen->start = hw->acl_tbl->first_entry;
 	scen->width = ICE_AQC_ACL_KEY_WIDTH_BYTES *
 		DIV_ROUND_UP(match_width, ICE_AQC_ACL_KEY_WIDTH_BYTES);
 	scen->num_entry = num_entries;
 
 	status = ice_acl_alloc_partition(hw, scen);
-	if (status) {
-		devm_kfree(ice_hw_to_dev(hw), scen);
-		return status;
-	}
+	if (status)
+		goto out;
 
 	memset(&scen_buf, 0, sizeof(scen_buf));
 
@@ -793,12 +790,12 @@ ice_acl_create_scen(struct ice_hw *hw, u16 match_width, u16 num_entries,
 	/* set the chunk mask for the tcams */
 	ice_acl_set_scen_chnk_msk(&scen_buf, scen);
 
-	/* set the TCAM select and start_cmp and start_stop bits */
+	/* set the TCAM select and start_cmp and start_set bits */
 	k = first_tcam;
 	/* set the START_SET bit at the beginning of the stack */
 	scen_buf.tcam_cfg[k].start_cmp_set |= ICE_AQC_ACL_ALLOC_SCE_START_SET;
 	while (k <= last_tcam) {
-		u16 last_tcam_idx_cascade = cascade_cnt + k - 1;
+		u8 last_tcam_idx_cascade = cascade_cnt + k - 1;
 
 		/* set start_cmp for the first cascaded TCAM */
 		scen_buf.tcam_cfg[k].start_cmp_set |=
@@ -816,13 +813,22 @@ ice_acl_create_scen(struct ice_hw *hw, u16 match_width, u16 num_entries,
 		k = i;
 	}
 
+	/* We need to set the start_cmp bit for the unused TCAMs. */
+	i = 0;
+	while (i < first_tcam)
+		scen_buf.tcam_cfg[i++].start_cmp_set =
+					ICE_AQC_ACL_ALLOC_SCE_START_CMP;
+
+	i = last_tcam + 1;
+	while (i < ICE_AQC_ACL_SLICES)
+		scen_buf.tcam_cfg[i++].start_cmp_set =
+					ICE_AQC_ACL_ALLOC_SCE_START_CMP;
+
 	status = ice_aq_alloc_acl_scen(hw, scen_id, &scen_buf, NULL);
 	if (status) {
-		ice_debug(hw, ICE_DBG_ACL,
-			  "AQ allocation of ACL scenario failed. status: %d\n",
+		ice_debug(hw, ICE_DBG_ACL, "AQ allocation of ACL scenario failed. status: %d\n",
 			  status);
-		devm_kfree(ice_hw_to_dev(hw), scen);
-		return status;
+		goto out;
 	}
 
 	scen->id = *scen_id;
@@ -830,9 +836,60 @@ ice_acl_create_scen(struct ice_hw *hw, u16 match_width, u16 num_entries,
 	ice_acl_init_entry(scen);
 	list_add(&scen->list_entry, &hw->acl_tbl->scens);
 
+out:
+	if (status)
+		devm_kfree(ice_hw_to_dev(hw), scen);
+
 	return status;
 }
 
+/**
+ * ice_acl_destroy_scen - Destroy an ACL scenario
+ * @hw: pointer to the HW struct
+ * @scen_id: ID of the remove scenario
+ */
+static enum ice_status ice_acl_destroy_scen(struct ice_hw *hw, u16 scen_id)
+{
+	struct ice_acl_scen *scen, *tmp_scen;
+	struct ice_flow_prof *p, *tmp;
+	enum ice_status status;
+
+	if (hw->dcf_acl_enabled)
+		return ICE_ERR_IN_USE;
+
+	if (!hw->acl_tbl)
+		return ICE_ERR_DOES_NOT_EXIST;
+
+	/* Remove profiles that use "scen_id" scenario */
+	list_for_each_entry_safe(p, tmp, &hw->fl_profs[ICE_BLK_ACL], l_entry)
+		if (p->cfg.scen && p->cfg.scen->id == scen_id) {
+			status = ice_flow_rem_prof(hw, ICE_BLK_ACL, p->id);
+			if (status) {
+				ice_debug(hw, ICE_DBG_ACL, "ice_flow_rem_prof failed. status: %d\n",
+					  status);
+				return status;
+			}
+		}
+
+	/* Call the AQ command to destroy the targeted scenario */
+	status = ice_aq_dealloc_acl_scen(hw, scen_id, NULL);
+	if (status) {
+		ice_debug(hw, ICE_DBG_ACL, "AQ de-allocation of scenario failed. status: %d\n",
+			  status);
+		return status;
+	}
+
+	/* Remove scenario from hw->acl_tbl->scens */
+	list_for_each_entry_safe(scen, tmp_scen, &hw->acl_tbl->scens,
+				 list_entry)
+		if (scen->id == scen_id) {
+			list_del(&scen->list_entry);
+			devm_kfree(ice_hw_to_dev(hw), scen);
+		}
+
+	return 0;
+}
+
 /**
  * ice_acl_destroy_tbl - Destroy a previously created LEM table for ACL
  * @hw: pointer to the HW struct
@@ -845,11 +902,17 @@ enum ice_status ice_acl_destroy_tbl(struct ice_hw *hw)
 	enum ice_status status;
 	u8 i;
 
+	if (hw->dcf_acl_enabled)
+		return ICE_ERR_IN_USE;
+
 	if (!hw->acl_tbl)
 		return ICE_ERR_DOES_NOT_EXIST;
 
-	/* Mark all the created scenario's TCAM to stop the packet lookup */
-	list_for_each_entry(pos_scen, &hw->acl_tbl->scens, list_entry) {
+	/* Mark all the created scenario's TCAM to stop the packet lookup and
+	 * delete them afterward
+	 */
+	list_for_each_entry_safe(pos_scen, tmp_scen, &hw->acl_tbl->scens,
+				 list_entry) {
 		status = ice_aq_query_acl_scen(hw, pos_scen->id, &buf, NULL);
 		if (status) {
 			ice_debug(hw, ICE_DBG_ACL, "ice_aq_query_acl_scen() failed. status: %d\n",
@@ -857,8 +920,14 @@ enum ice_status ice_acl_destroy_tbl(struct ice_hw *hw)
 			return status;
 		}
 
-		for (i = 0; i < ICE_AQC_ACL_SLICES; i++)
-			buf.tcam_cfg[i].chnk_msk = 0x0;
+		for (i = 0; i < ICE_AQC_ACL_SLICES; i++) {
+			buf.tcam_cfg[i].chnk_msk = 0;
+			buf.tcam_cfg[i].start_cmp_set =
+					ICE_AQC_ACL_ALLOC_SCE_START_CMP;
+		}
+
+		for (i = 0; i < ICE_AQC_MAX_ACTION_MEMORIES; i++)
+			buf.act_mem_cfg[i] = 0;
 
 		status = ice_aq_update_acl_scen(hw, pos_scen->id, &buf, NULL);
 		if (status) {
@@ -866,24 +935,23 @@ enum ice_status ice_acl_destroy_tbl(struct ice_hw *hw)
 				  status);
 			return status;
 		}
+
+		status = ice_acl_destroy_scen(hw, pos_scen->id);
+		if (status) {
+			ice_debug(hw, ICE_DBG_ACL, "deletion of scenario failed. status: %d\n",
+				  status);
+			return status;
+		}
 	}
 
-	/* call the aq command to destroy the ACL table with these values */
+	/* call the AQ command to destroy the ACL table */
 	status = ice_aq_dealloc_acl_tbl(hw, hw->acl_tbl->id, &resp_buf, NULL);
-
 	if (status) {
-		ice_debug(hw, ICE_DBG_ACL,
-			  "AQ de-allocation of ACL failed. status: %d\n",
+		ice_debug(hw, ICE_DBG_ACL, "AQ de-allocation of ACL failed. status: %d\n",
 			  status);
 		return status;
 	}
 
-	list_for_each_entry_safe(pos_scen, tmp_scen, &hw->acl_tbl->scens,
-				 list_entry) {
-		list_del(&pos_scen->list_entry);
-		devm_kfree(ice_hw_to_dev(hw), pos_scen);
-	}
-
 	devm_kfree(ice_hw_to_dev(hw), hw->acl_tbl);
 	hw->acl_tbl = NULL;
 
@@ -894,7 +962,7 @@ enum ice_status ice_acl_destroy_tbl(struct ice_hw *hw)
  * ice_acl_add_entry - Add a flow entry to an ACL scenario
  * @hw: pointer to the HW struct
  * @scen: scenario to add the entry to
- * @prior: priority level of the entry being added
+ * @prio: priority level of the entry being added
  * @keys: buffer of the value of the key to be programmed to the ACL entry
  * @inverts: buffer of the value of the key inverts to be programmed
  * @acts: pointer to a buffer containing formatted actions
@@ -908,17 +976,21 @@ enum ice_status ice_acl_destroy_tbl(struct ice_hw *hw)
  */
 enum ice_status
 ice_acl_add_entry(struct ice_hw *hw, struct ice_acl_scen *scen,
-		  enum ice_acl_entry_prior prior, u8 *keys, u8 *inverts,
+		  enum ice_acl_entry_prio prio, u8 *keys, u8 *inverts,
 		  struct ice_acl_act_entry *acts, u8 acts_cnt, u16 *entry_idx)
 {
-	u16 i, entry_tcam, num_cscd, idx;
+	u8 i, entry_tcam, num_cscd, offset;
 	struct ice_aqc_acl_data buf;
 	enum ice_status status = 0;
+	u16 idx;
+
+	if (hw->dcf_acl_enabled)
+		return ICE_ERR_IN_USE;
 
 	if (!scen)
 		return ICE_ERR_DOES_NOT_EXIST;
 
-	*entry_idx = ice_acl_scen_assign_entry_idx(scen, prior);
+	*entry_idx = ice_acl_scen_assign_entry_idx(scen, prio);
 	if (*entry_idx >= scen->num_entry) {
 		*entry_idx = 0;
 		return ICE_ERR_MAX_LIMIT;
@@ -926,15 +998,12 @@ ice_acl_add_entry(struct ice_hw *hw, struct ice_acl_scen *scen,
 
 	/* Determine number of cascaded TCAMs */
 	num_cscd = DIV_ROUND_UP(scen->width, ICE_AQC_ACL_KEY_WIDTH_BYTES);
-	entry_tcam = hw->acl_tbl->first_tcam +
-		ICE_ACL_TBL_TCAM_IDX(scen->start) +
-		(ICE_ACL_TBL_TCAM_ENTRY_IDX(scen->start) + *entry_idx) /
-		ICE_AQC_ACL_TCAM_DEPTH * num_cscd;
 
+	entry_tcam = ICE_ACL_TBL_TCAM_IDX(scen->start);
 	idx = ICE_ACL_TBL_TCAM_ENTRY_IDX(scen->start + *entry_idx);
 
 	memset(&buf, 0, sizeof(buf));
-	for (i = num_cscd - 1; i < num_cscd; i--) {
+	for (i = 0; i < num_cscd; i++) {
 		/* If the key spans more than one TCAM in the case of cascaded
 		 * TCAMs, the key and key inverts need to be properly split
 		 * among TCAMs.E.g.bytes 0 - 4 go to an index in the first TCAM
@@ -945,17 +1014,17 @@ ice_acl_add_entry(struct ice_hw *hw, struct ice_acl_scen *scen,
 		 * be programmed first; the TCAM entry of the leftmost TCAM
 		 * should be programmed last.
 		 */
+		offset = num_cscd - i - 1;
 		memcpy(&buf.entry_key.val,
-		       &keys[i * sizeof(buf.entry_key.val)],
+		       &keys[offset * sizeof(buf.entry_key.val)],
 		       sizeof(buf.entry_key.val));
 		memcpy(&buf.entry_key_invert.val,
-		       &inverts[i * sizeof(buf.entry_key_invert.val)],
+		       &inverts[offset * sizeof(buf.entry_key_invert.val)],
 		       sizeof(buf.entry_key_invert.val));
-		status = ice_aq_program_acl_entry(hw, entry_tcam + i, idx, &buf,
-						  NULL);
+		status = ice_aq_program_acl_entry(hw, entry_tcam + offset, idx,
+						  &buf, NULL);
 		if (status) {
-			ice_debug(hw, ICE_DBG_ACL,
-				  "aq program acl entry failed status: %d\n",
+			ice_debug(hw, ICE_DBG_ACL, "aq program acl entry failed status: %d\n",
 				  status);
 			goto out;
 		}
@@ -988,9 +1057,13 @@ ice_acl_prog_act(struct ice_hw *hw, struct ice_acl_scen *scen,
 		 struct ice_acl_act_entry *acts, u8 acts_cnt,
 		 u16 entry_idx)
 {
-	u16 entry_tcam, num_cscd, i, idx, actx_idx = 0;
+	u8 entry_tcam, num_cscd, i, actx_idx = 0;
 	struct ice_aqc_actpair act_buf;
 	enum ice_status status = 0;
+	u16 idx;
+
+	if (hw->dcf_acl_enabled)
+		return ICE_ERR_IN_USE;
 
 	if (entry_idx >= scen->num_entry)
 		return ICE_ERR_MAX_LIMIT;
@@ -999,15 +1072,11 @@ ice_acl_prog_act(struct ice_hw *hw, struct ice_acl_scen *scen,
 
 	/* Determine number of cascaded TCAMs */
 	num_cscd = DIV_ROUND_UP(scen->width, ICE_AQC_ACL_KEY_WIDTH_BYTES);
-	entry_tcam = hw->acl_tbl->first_tcam +
-		ICE_ACL_TBL_TCAM_IDX(scen->start) +
-		(ICE_ACL_TBL_TCAM_ENTRY_IDX(scen->start) + entry_idx) /
-		ICE_AQC_ACL_TCAM_DEPTH * num_cscd;
 
+	entry_tcam = ICE_ACL_TBL_TCAM_IDX(scen->start);
 	idx = ICE_ACL_TBL_TCAM_ENTRY_IDX(scen->start + entry_idx);
 
-	i = find_first_bit(scen->act_mem_bitmap, ICE_AQC_MAX_ACTION_MEMORIES);
-	while (i < ICE_AQC_MAX_ACTION_MEMORIES) {
+	for_each_set_bit(i, scen->act_mem_bitmap, ICE_AQC_MAX_ACTION_MEMORIES) {
 		struct ice_acl_act_mem *mem = &hw->acl_tbl->act_mems[i];
 
 		if (actx_idx >= acts_cnt)
@@ -1025,16 +1094,12 @@ ice_acl_prog_act(struct ice_hw *hw, struct ice_acl_scen *scen,
 			status = ice_aq_program_actpair(hw, i, idx, &act_buf,
 							NULL);
 			if (status) {
-				ice_debug(hw, ICE_DBG_ACL,
-					  "program actpair failed status: %d\n",
+				ice_debug(hw, ICE_DBG_ACL, "program actpair failed status: %d\n",
 					  status);
 				break;
 			}
 			actx_idx++;
 		}
-
-		i = find_next_bit(scen->act_mem_bitmap,
-				  ICE_AQC_MAX_ACTION_MEMORIES, i + 1);
 	}
 
 	if (!status && actx_idx < acts_cnt)
@@ -1052,10 +1117,14 @@ ice_acl_prog_act(struct ice_hw *hw, struct ice_acl_scen *scen,
 enum ice_status
 ice_acl_rem_entry(struct ice_hw *hw, struct ice_acl_scen *scen, u16 entry_idx)
 {
-	u16 entry_tcam, num_cscd, idx, i;
 	struct ice_aqc_actpair act_buf;
 	struct ice_aqc_acl_data buf;
+	u8 entry_tcam, num_cscd, i;
 	enum ice_status status = 0;
+	u16 idx;
+
+	if (hw->dcf_acl_enabled)
+		return ICE_ERR_IN_USE;
 
 	if (!scen)
 		return ICE_ERR_DOES_NOT_EXIST;
@@ -1068,11 +1137,8 @@ ice_acl_rem_entry(struct ice_hw *hw, struct ice_acl_scen *scen, u16 entry_idx)
 
 	/* Determine number of cascaded TCAMs */
 	num_cscd = DIV_ROUND_UP(scen->width, ICE_AQC_ACL_KEY_WIDTH_BYTES);
-	entry_tcam = hw->acl_tbl->first_tcam +
-		ICE_ACL_TBL_TCAM_IDX(scen->start) +
-		(ICE_ACL_TBL_TCAM_ENTRY_IDX(scen->start) + entry_idx) /
-		ICE_AQC_ACL_TCAM_DEPTH * num_cscd;
 
+	entry_tcam = ICE_ACL_TBL_TCAM_IDX(scen->start);
 	idx = ICE_ACL_TBL_TCAM_ENTRY_IDX(scen->start + entry_idx);
 
 	/* invalidate the flow entry */
@@ -1081,14 +1147,13 @@ ice_acl_rem_entry(struct ice_hw *hw, struct ice_acl_scen *scen, u16 entry_idx)
 		status = ice_aq_program_acl_entry(hw, entry_tcam + i, idx, &buf,
 						  NULL);
 		if (status)
-			ice_debug(hw, ICE_DBG_ACL,
-				  "aq program acl entry failed status: %d\n",
+			ice_debug(hw, ICE_DBG_ACL, "AQ program ACL entry failed status: %d\n",
 				  status);
 	}
 
 	memset(&act_buf, 0, sizeof(act_buf));
-	i = find_first_bit(scen->act_mem_bitmap, ICE_AQC_MAX_ACTION_MEMORIES);
-	while (i < ICE_AQC_MAX_ACTION_MEMORIES) {
+
+	for_each_set_bit(i, scen->act_mem_bitmap, ICE_AQC_MAX_ACTION_MEMORIES) {
 		struct ice_acl_act_mem *mem = &hw->acl_tbl->act_mems[i];
 
 		if (mem->member_of_tcam >= entry_tcam &&
@@ -1097,13 +1162,9 @@ ice_acl_rem_entry(struct ice_hw *hw, struct ice_acl_scen *scen, u16 entry_idx)
 			status = ice_aq_program_actpair(hw, i, idx, &act_buf,
 							NULL);
 			if (status)
-				ice_debug(hw, ICE_DBG_ACL,
-					  "program actpair failed.status: %d\n",
+				ice_debug(hw, ICE_DBG_ACL, "program actpair failed status: %d\n",
 					  status);
 		}
-
-		i = find_next_bit(scen->act_mem_bitmap,
-				  ICE_AQC_MAX_ACTION_MEMORIES, i + 1);
 	}
 
 	ice_acl_scen_free_entry_idx(scen, entry_idx);
@@ -1112,48 +1173,20 @@ ice_acl_rem_entry(struct ice_hw *hw, struct ice_acl_scen *scen, u16 entry_idx)
 }
 
 /**
- * ice_acl_destroy_scen - Destroy an ACL scenario
+ * ice_is_acl_empty - Check if any entry exists
  * @hw: pointer to the HW struct
- * @scen_id: ID of the remove scenario
  */
-enum ice_status ice_acl_destroy_scen(struct ice_hw *hw, u16 scen_id)
+bool ice_is_acl_empty(struct ice_hw *hw)
 {
 	struct ice_acl_scen *scen, *tmp_scen;
-	struct ice_flow_prof *p, *tmp;
-	enum ice_status status;
 
 	if (!hw->acl_tbl)
-		return ICE_ERR_DOES_NOT_EXIST;
+		return false;
 
-	/* Remove profiles that use "scen_id" scenario */
-	list_for_each_entry_safe(p, tmp, &hw->fl_profs[ICE_BLK_ACL], l_entry)
-		if (p->cfg.scen && p->cfg.scen->id == scen_id) {
-			status = ice_flow_rem_prof(hw, ICE_BLK_ACL, p->id);
-			if (status) {
-				ice_debug(hw, ICE_DBG_ACL,
-					  "ice_flow_rem_prof failed. status: %d\n",
-					  status);
-				goto exit;
-			}
-		}
-
-	/* Call the aq command to destroy the targeted scenario */
-	status = ice_aq_dealloc_acl_scen(hw, scen_id, NULL);
-
-	if (status) {
-		ice_debug(hw, ICE_DBG_ACL,
-			  "AQ de-allocation of scenario failed. status: %d\n",
-			  status);
-		goto exit;
-	}
-
-	/* Remove scenario from hw->acl_tbl->scens */
 	list_for_each_entry_safe(scen, tmp_scen, &hw->acl_tbl->scens,
 				 list_entry)
-		if (scen->id == scen_id) {
-			list_del(&scen->list_entry);
-			devm_kfree(ice_hw_to_dev(hw), scen);
-		}
-exit:
-	return status;
+		if (!bitmap_empty(scen->entry_bitmap, ICE_MAX_ACL_TCAM_ENTRY))
+			return false;
+
+	return true;
 }
