@@ -4,6 +4,7 @@
 /* Inter-Driver Communication */
 #include "ice.h"
 #include "ice_lib.h"
+#include "ice_fltr.h"
 #include "ice_dcb_lib.h"
 
 DEFINE_IDA(ice_peer_index_ida);
@@ -12,6 +13,25 @@ DEFINE_IDA(ice_peer_index_ida);
 
 static struct mfd_cell ice_mfd_cells[] = ASSIGN_PEER_INFO;
 
+/**
+ * ice_is_vsi_state_nominal
+ * @vsi: pointer to the VSI struct
+ *
+ * returns true if VSI state is nominal, false otherwise
+ */
+static bool ice_is_vsi_state_nominal(struct ice_vsi *vsi)
+{
+	if (!vsi)
+		return false;
+
+	if (test_bit(ICE_VSI_DOWN, vsi->state) ||
+	    test_bit(ICE_VSI_NEEDS_RESTART, vsi->state) ||
+	    test_bit(ICE_VSI_BUSY, vsi->state))
+		return false;
+
+	return true;
+}
+
 
 /**
  * ice_peer_state_change - manage state machine for peer
@@ -33,6 +53,9 @@ static struct mfd_cell ice_mfd_cells[] = ASSIGN_PEER_INFO;
  *					 +	     +
  *				      PREPPED	     +
  *					 +---------->+
+ *
+ * NOTE: there is an error condition that can take a peer from OPENING
+ * to REMOVED.
  */
 static void
 ice_peer_state_change(struct ice_peer_dev_int *peer_dev, long new_state,
@@ -120,7 +143,8 @@ ice_peer_state_change(struct ice_peer_dev_int *peer_dev, long new_state,
 		if (test_and_clear_bit(ICE_PEER_DEV_STATE_PREP_RST,
 				       peer_dev->state)) {
 			set_bit(ICE_PEER_DEV_STATE_CLOSING, peer_dev->state);
-			dev_warn(dev, "WARN: Peer state PREP_RST to _CLOSING\n");
+			dev_warn(dev,
+				 "WARN: Peer state _PREP_RST to _CLOSING\n");
 		}
 		break;
 	case ICE_PEER_DEV_STATE_CLOSED:
@@ -140,6 +164,11 @@ ice_peer_state_change(struct ice_peer_dev_int *peer_dev, long new_state,
 			/* Clear registration for events when peer removed */
 			bitmap_zero(peer_dev->events, ICE_PEER_DEV_STATE_NBITS);
 		}
+		if (test_and_clear_bit(ICE_PEER_DEV_STATE_OPENING,
+				       peer_dev->state)) {
+			set_bit(ICE_PEER_DEV_STATE_REMOVED, peer_dev->state);
+			dev_warn(dev, "Peer failed to open, set to _REMOVED");
+		}
 		break;
 	default:
 		break;
@@ -147,6 +176,8 @@ ice_peer_state_change(struct ice_peer_dev_int *peer_dev, long new_state,
 
 	if (!locked)
 		mutex_unlock(&peer_dev->peer_dev_state_mutex);
+
+	put_device(dev);
 }
 
 /**
@@ -170,9 +201,9 @@ int ice_peer_close(struct ice_peer_dev_int *peer_dev_int, void *data)
 		return 0;
 	pf = pci_get_drvdata(peer_dev->pdev);
 
-	if (test_bit(__ICE_DOWN, pf->state) ||
-	    test_bit(__ICE_SUSPENDED, pf->state) ||
-	    test_bit(__ICE_NEEDS_RESTART, pf->state))
+	if (test_bit(ICE_DOWN, pf->state) ||
+	    test_bit(ICE_SUSPENDED, pf->state) ||
+	    test_bit(ICE_NEEDS_RESTART, pf->state))
 		return 0;
 
 	mutex_lock(&peer_dev_int->peer_dev_state_mutex);
@@ -181,6 +212,7 @@ int ice_peer_close(struct ice_peer_dev_int *peer_dev_int, void *data)
 	if (test_bit(ICE_PEER_DEV_STATE_CLOSED, peer_dev_int->state) ||
 	    test_bit(ICE_PEER_DEV_STATE_CLOSING, peer_dev_int->state) ||
 	    test_bit(ICE_PEER_DEV_STATE_OPENING, peer_dev_int->state) ||
+	    test_bit(ICE_PEER_DEV_STATE_PROBED, peer_dev_int->state) ||
 	    test_bit(ICE_PEER_DEV_STATE_REMOVED, peer_dev_int->state))
 		goto peer_close_out;
 
@@ -233,12 +265,17 @@ int ice_close_peer_for_reset(struct ice_peer_dev_int *peer_dev_int, void *data)
 	enum ice_reset_req reset;
 
 	peer_dev = ice_get_peer_dev(peer_dev_int);
-	if (!ice_validate_peer_dev(peer_dev))
+	if (!ice_validate_peer_dev(peer_dev) ||
+	    (!test_bit(ICE_PEER_DEV_STATE_OPENED, peer_dev_int->state) &&
+	     !test_bit(ICE_PEER_DEV_STATE_PREPPED, peer_dev_int->state)))
 		return 0;
 
 	reset = *(enum ice_reset_req *)data;
 
 	switch (reset) {
+	case ICE_RESET_EMPR:
+		peer_dev_int->rst_type = ICE_REASON_EMPR_REQ;
+		break;
 	case ICE_RESET_GLOBR:
 		peer_dev_int->rst_type = ICE_REASON_GLOBR_REQ;
 		break;
@@ -301,7 +338,7 @@ ice_check_peer_for_events(struct ice_peer_dev_int *src_peer_int, void *data)
 	const struct ice_peer_ops *p_ops = new_peer->peer_ops;
 	struct ice_peer_dev_int *new_peer_int;
 	struct ice_peer_dev *src_peer;
-	int i;
+	unsigned long i;
 
 	src_peer = ice_get_peer_dev(src_peer_int);
 	if (!ice_validate_peer_dev(new_peer) ||
@@ -332,7 +369,7 @@ int
 ice_for_each_peer(struct ice_pf *pf, void *data,
 		  int (*fn)(struct ice_peer_dev_int *, void *))
 {
-	int i;
+	unsigned int i;
 
 	if (!pf->peers)
 		return 0;
@@ -417,12 +454,17 @@ ice_finish_init_peer_device(struct ice_peer_dev_int *peer_dev_int,
 		ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_OPENING,
 				      true);
 		ret = peer_dev->peer_ops->open(peer_dev);
-		if (ret) {
+		if (ret == -EAGAIN) {
 			dev_err(dev, "Peer %d failed to open\n",
 				peer_dev->peer_dev_id);
 			ice_peer_state_change(peer_dev_int,
 					      ICE_PEER_DEV_STATE_PROBED, true);
 			goto init_unlock;
+		} else if (ret) {
+			ice_peer_state_change(peer_dev_int,
+					      ICE_PEER_DEV_STATE_REMOVED, true);
+			peer_dev->peer_ops = NULL;
+			goto init_unlock;
 		}
 
 		ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_OPENED,
@@ -432,6 +474,22 @@ ice_finish_init_peer_device(struct ice_peer_dev_int *peer_dev_int,
 		ice_check_peer_drv_for_events(peer_dev);
 	}
 
+	if (test_bit(ICE_PEER_DEV_STATE_PREPPED, peer_dev_int->state)) {
+		enum ice_close_reason reason = ICE_REASON_CORER_REQ;
+		int i;
+
+		ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_CLOSING,
+				      true);
+		for (i = 0; i < ICE_EVENT_NBITS; i++)
+			bitmap_zero(peer_dev_int->current_events[i].type,
+				    ICE_EVENT_NBITS);
+
+		peer_dev->peer_ops->close(peer_dev, reason);
+
+		ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_CLOSED,
+				      true);
+	}
+
 init_unlock:
 	mutex_unlock(&peer_dev_int->peer_dev_state_mutex);
 
@@ -507,7 +565,7 @@ ice_unroll_peer(struct ice_peer_dev_int *peer_dev_int,
 	struct ice_pf *pf;
 
 	peer_dev = ice_get_peer_dev(peer_dev_int);
-	if (!peer_dev)
+	if (!peer_dev || !peer_dev->pdev)
 		return 0;
 
 	pf = pci_get_drvdata(peer_dev->pdev);
@@ -516,6 +574,10 @@ ice_unroll_peer(struct ice_peer_dev_int *peer_dev_int,
 
 	if (peer_dev_int->ice_peer_wq)
 		destroy_workqueue(peer_dev_int->ice_peer_wq);
+
+	if (peer_dev_int->peer_drv_int)
+		devm_kfree(ice_pf_to_dev(pf), peer_dev_int->peer_drv_int);
+
 	devm_kfree(ice_pf_to_dev(pf), peer_dev_int);
 
 	return 0;
@@ -529,7 +591,7 @@ ice_unroll_peer(struct ice_peer_dev_int *peer_dev_int,
 void ice_peer_refresh_msix(struct ice_pf *pf)
 {
 	struct ice_peer_dev *peer;
-	int i;
+	unsigned int i;
 
 	if (!pf->peers)
 		return;
@@ -599,6 +661,9 @@ ice_peer_alloc_rdma_qsets(struct ice_peer_dev *peer_dev, struct ice_res *res,
 	pf = pci_get_drvdata(peer_dev->pdev);
 	dev = ice_pf_to_dev(pf);
 
+	if (!test_bit(ICE_FLAG_IWARP_ENA, pf->flags))
+		return -EINVAL;
+
 	if (res->cnt_req > ICE_MAX_TXQ_PER_TXQG)
 		return -EINVAL;
 
@@ -895,7 +960,8 @@ ice_peer_report_state_change(struct ice_peer_dev *peer_dev,
 {
 	struct ice_peer_dev_int *peer_dev_int;
 	struct ice_peer_drv_int *peer_drv_int;
-	int e_type, drv_event = 0;
+	unsigned int e_type;
+	int drv_event = 0;
 	struct ice_pf *pf;
 
 	if (!ice_validate_peer_dev(peer_dev) || !event)
@@ -923,12 +989,13 @@ ice_peer_report_state_change(struct ice_peer_dev *peer_dev,
 
 	/* Check for peer_dev events */
 	case ICE_EVENT_API_CHANGE:
-		if (event->info.api_rdy)
+		if (event->info.api_rdy) {
 			set_bit(ICE_PEER_DEV_STATE_API_RDY,
 				peer_dev_int->state);
-		else
+		} else {
 			clear_bit(ICE_PEER_DEV_STATE_API_RDY,
 				  peer_dev_int->state);
+		}
 		break;
 
 	default:
@@ -955,7 +1022,7 @@ ice_peer_report_state_change(struct ice_peer_dev *peer_dev,
  */
 static int ice_peer_unregister(struct ice_peer_dev *peer_dev)
 {
-	enum ice_close_reason reason = ICE_REASON_PEER_DEV_UNINIT;
+	enum ice_close_reason reason = ICE_REASON_PEER_DRV_UNREG;
 	struct ice_peer_dev_int *peer_dev_int;
 	struct ice_pf *pf;
 	int ret;
@@ -973,12 +1040,21 @@ static int ice_peer_unregister(struct ice_peer_dev *peer_dev)
 	if (ret)
 		return ret;
 
+	switch (peer_dev->peer_dev_id) {
+	case ICE_PEER_RDMA_ID:
+		pf->rdma_peer = NULL;
+#ifdef HAVE_NETDEV_UPPER_INFO
+		if (pf->lag)
+			ice_enable_lag(pf->lag);
+#endif /* HAVE_NETDEV_UPPER_INFO */
+		break;
+	default:
+		break;
+	}
+
 	peer_dev->peer_ops = NULL;
 
 	ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_REMOVED, false);
-
-	module_put(THIS_MODULE);
-
 	return 0;
 }
 
@@ -1016,6 +1092,7 @@ static int ice_peer_register(struct ice_peer_dev *peer_dev)
 		return -EINVAL;
 	}
 
+
 	if (peer_drv->ver.major != ICE_PEER_MAJOR_VER ||
 	    peer_drv->ver.minor != ICE_PEER_MINOR_VER) {
 		pr_err("failed to register due to version mismatch:\n");
@@ -1025,6 +1102,18 @@ static int ice_peer_register(struct ice_peer_dev *peer_dev)
 		       ICE_PEER_MINOR_VER, peer_drv->ver.minor);
 		return -EINVAL;
 	}
+#ifdef HAVE_NETDEV_UPPER_INFO
+	if (peer_dev->peer_dev_id == ICE_PEER_RDMA_ID) {
+		struct ice_pf *pf = pf = pci_get_drvdata(peer_dev->pdev);
+
+		if (pf->lag) {
+			if (pf->lag->bonded)
+				return -EINVAL;
+
+			ice_disable_lag(pf->lag);
+		}
+	}
+#endif /* HAVE_NETDEV_UPPER_INFO */
 
 	peer_dev_int = peer_to_ice_dev_int(peer_dev);
 	peer_drv_int = peer_dev_int->peer_drv_int;
@@ -1037,14 +1126,10 @@ static int ice_peer_register(struct ice_peer_dev *peer_dev)
 
 	ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_PROBED, false);
 
-	if (!try_module_get(THIS_MODULE)) {
-		pr_err("Failed to increment module use count\n");
-		return -EINVAL;
-	}
-
 	return 0;
 }
 
+
 /**
  * ice_peer_request_reset - accept request from peer to perform a reset
  * @peer_dev: peer device that is request a reset
@@ -1086,7 +1171,6 @@ ice_peer_request_reset(struct ice_peer_dev *peer_dev,
  */
 static int ice_peer_is_vsi_ready(struct ice_peer_dev *peer_dev)
 {
-	DECLARE_BITMAP(check_bits, __ICE_STATE_NBITS) = { 0 };
 	struct ice_netdev_priv *np;
 	struct ice_vsi *vsi;
 
@@ -1095,7 +1179,8 @@ static int ice_peer_is_vsi_ready(struct ice_peer_dev *peer_dev)
 	 * as parameters.
 	 */
 
-	if (!ice_validate_peer_dev(peer_dev))
+	if (!peer_dev || !peer_dev->pdev || !pci_get_drvdata(peer_dev->pdev) ||
+	    !peer_to_ice_dev_int(peer_dev))
 		return 0;
 
 	if (!peer_dev->netdev)
@@ -1103,14 +1188,8 @@ static int ice_peer_is_vsi_ready(struct ice_peer_dev *peer_dev)
 
 	np = netdev_priv(peer_dev->netdev);
 	vsi = np->vsi;
-	if (!vsi)
-		return 0;
 
-	bitmap_set(check_bits, 0, __ICE_STATE_NOMINAL_CHECK_BITS);
-	if (bitmap_intersects(vsi->state, check_bits, __ICE_STATE_NBITS))
-		return 0;
-
-	return 1;
+	return ice_is_vsi_state_nominal(vsi);
 }
 
 /**
@@ -1153,8 +1232,8 @@ ice_peer_update_vsi_filter(struct ice_peer_dev *peer_dev,
 }
 
 /**
- * ice_peer_vc_send - send a virt channel message from RDMA peer
- * @peer_dev: pointer to RDMA peer dev
+ * ice_peer_vc_send - send a virt channel message from a peer
+ * @peer_dev: pointer to a peer dev
  * @vf_id: the absolute VF ID of recipient of message
  * @msg: pointer to message contents
  * @len: len of message
@@ -1171,16 +1250,25 @@ ice_peer_vc_send(struct ice_peer_dev *peer_dev, u32 vf_id, u8 *msg, u16 len)
 		return -ENOMEM;
 
 	pf = pci_get_drvdata(peer_dev->pdev);
-	if (vf_id >= pf->num_alloc_vfs || len > ICE_AQ_MAX_BUF_LEN)
+	if (len > ICE_AQ_MAX_BUF_LEN)
 		return -EINVAL;
 
-	/* VIRTCHNL_OP_IWARP is being used for RoCEv2 msg also */
-	err = ice_aq_send_msg_to_vf(&pf->hw, vf_id, VIRTCHNL_OP_IWARP, 0, msg,
-				    len, NULL);
-	if (err)
-		dev_err(ice_pf_to_dev(pf), "Unable to send RDMA msg to VF, error %d\n",
-			err);
+	switch (peer_dev->peer_drv->driver_id) {
+	case ICE_PEER_RDMA_DRIVER:
+		if (vf_id >= pf->num_alloc_vfs)
+			return -ENODEV;
 
+		/* VIRTCHNL_OP_IWARP is being used for RoCEv2 msg also */
+		err = ice_aq_send_msg_to_vf(&pf->hw, vf_id, VIRTCHNL_OP_IWARP,
+					    0, msg, len, NULL);
+		break;
+	default:
+		err = ICE_ERR_DEVICE_NOT_SUPPORTED;
+	}
+
+	if (err)
+		dev_err(ice_pf_to_dev(pf),
+			"Unable to send msg to VF, error %d\n", err);
 	return err;
 }
 
@@ -1214,7 +1302,7 @@ static int ice_reserve_peer_qvector(struct ice_pf *pf)
 		if (index < 0)
 			return index;
 		pf->num_avail_sw_msix -= pf->num_rdma_msix;
-		pf->rdma_base_vector = index;
+		pf->rdma_base_vector = (u16)index;
 	}
 	return 0;
 }
@@ -1245,12 +1333,25 @@ static void ice_peer_close_task(struct work_struct *work)
 	 */
 	mutex_lock(&peer_dev_int->peer_dev_state_mutex);
 
-	ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_CLOSING, true);
+	/* Only allow a close to go to the peer if they are in a state
+	 * to accept it. The last state of PREP_RST is a special case
+	 * that will not normally happen, but it is up to the peer
+	 * to handle it correctly.
+	 */
+	if (test_bit(ICE_PEER_DEV_STATE_OPENED, peer_dev_int->state) ||
+	    test_bit(ICE_PEER_DEV_STATE_PREPPED, peer_dev_int->state) ||
+	    test_bit(ICE_PEER_DEV_STATE_PREP_RST, peer_dev_int->state)) {
 
-	if (peer_dev->peer_ops->close)
-		peer_dev->peer_ops->close(peer_dev, peer_dev_int->rst_type);
+		ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_CLOSING,
+				      true);
 
-	ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_CLOSED, true);
+		if (peer_dev->peer_ops->close)
+			peer_dev->peer_ops->close(peer_dev,
+						  peer_dev_int->rst_type);
+
+		ice_peer_state_change(peer_dev_int, ICE_PEER_DEV_STATE_CLOSED,
+				      true);
+	}
 
 	mutex_unlock(&peer_dev_int->peer_dev_state_mutex);
 }
@@ -1269,7 +1370,7 @@ int ice_init_peer_devices(struct ice_pf *pf)
 	struct pci_dev *pdev = pf->pdev;
 	struct device *dev = &pdev->dev;
 	int status = 0;
-	int i;
+	unsigned int i;
 
 	/* Reserve vector resources */
 	status = ice_reserve_peer_qvector(pf);
@@ -1368,6 +1469,7 @@ int ice_init_peer_devices(struct ice_pf *pf)
 				peer_dev->msix_count = pf->num_rdma_msix;
 				entry = &pf->msix_entries[pf->rdma_base_vector];
 			}
+			pf->rdma_peer = peer_dev;
 			break;
 		default:
 			break;
@@ -1388,8 +1490,10 @@ int ice_init_peer_devices(struct ice_pf *pf)
 
 	status = mfd_add_devices(dev, pf->peer_idx, ice_mfd_cells,
 				 ARRAY_SIZE(ice_mfd_cells), NULL, 0, NULL);
-	if (status)
+	if (status) {
 		dev_err(dev, "Failure adding MFD devs for peers: %d\n", status);
+		return status;
+	}
 
 	for (i = 0; i < ARRAY_SIZE(ice_mfd_cells); i++) {
 		snprintf(pf->peers[i]->plat_name, ICE_MAX_PEER_NAME, "%s.%d",
@@ -1397,8 +1501,11 @@ int ice_init_peer_devices(struct ice_pf *pf)
 			 pf->peer_idx + ice_mfd_cells[i].id);
 		dev = bus_find_device_by_name(&platform_bus_type, NULL,
 					      pf->peers[i]->plat_name);
-		dev_dbg(dev, "Peer Created: %s %d\n", pf->peers[i]->plat_name,
-			pf->peer_idx);
+		if (dev) {
+			dev_dbg(dev, "Peer Created: %s %d\n",
+				pf->peers[i]->plat_name, pf->peer_idx);
+			put_device(dev);
+		}
 	}
 
 	return status;
