@@ -2,7 +2,7 @@
  *
  * Definitions for packet ring
  *
- * 2004-2020 - ntop.org
+ * 2004-2021 - ntop.org
  *
  */
 
@@ -23,8 +23,8 @@
 #endif /* __KERNEL__ */
 
 /* Versioning */
-#define RING_VERSION                "7.8.0"
-#define RING_VERSION_NUM           0x070800
+#define RING_VERSION                "8.0.0"
+#define RING_VERSION_NUM           0x080000
 
 /* Increment whenever we change slot or packet header layout (e.g. we add/move a field) */
 #define RING_FLOWSLOT_VERSION          18
@@ -120,6 +120,12 @@
 #define PF_RING_ERROR_NOT_ENOUGH_MEMORY   -10
 #define PF_RING_ERROR_INVALID_STATUS      -11
 #define PF_RING_ERROR_RING_NOT_ENABLED    -12
+#define PF_RING_ERROR_BAD_IFNAME          -13
+#define PF_RING_ERROR_MOD_NOT_LOADED      -14
+#define PF_RING_ERROR_UNABLE_TO_MAP       -15
+#define PF_RING_ERROR_UNABLE_TO_GET_INFO  -16
+#define PF_RING_ERROR_MMAP_FAILURE        -17
+#define PF_RING_ERROR_INIT_FAILURE        -18
 
 #define REFLECTOR_NAME_LEN                 8
 
@@ -572,6 +578,8 @@ typedef struct {
   u_int16_t dst_port_low;
   u_int16_t dst_port_high; /* mode 1 only */
   u_int8_t l4_port_not; /* rule match if src_port_low/dst_port_low are defined and they do not match - mode 2 only */
+  u_int8_t steer_to_ring; /* steer matching traffic to the specified ring_id (instead of LB) */
+  u_int8_t ring_id; /* destination ring for action = pass */
 } __attribute__((packed))
 accolade_hw_rule;
 
@@ -1136,20 +1144,6 @@ struct cluster_referee {
 
 /* ************************************************* */
 
-typedef int (*do_handle_sw_filtering_hash_bucket)(struct pf_ring_socket *pfr,
-					       sw_filtering_hash_bucket* rule,
-					       u_char add_rule);
-
-typedef int (*do_add_packet_to_ring)(struct pf_ring_socket *pfr,
-				     u_int8_t real_skb,
-				     struct pfring_pkthdr *hdr, struct sk_buff *skb,
-				     int displ, u_int8_t parse_pkt_first);
-
-typedef int (*do_add_raw_packet_to_ring)(struct pf_ring_socket *pfr,
-					 struct pfring_pkthdr *hdr,
-					 u_char *data, u_int data_len,
-					 u_int8_t parse_pkt_first);
-
 typedef u_int32_t (*do_rehash_rss)(struct sk_buff *skb, struct pfring_pkthdr *hdr);
 
 /* ************************************************* */
@@ -1190,7 +1184,6 @@ struct pf_ring_socket {
   DECLARE_BITMAP(pf_dev_mask, MAX_NUM_DEV_IDX /* bits */);
   int ring_pid;
   u_int32_t ring_id;
-  char *appl_name; /* String that identifies the application bound to the socket */
   packet_direction direction; /* Specify the capture direction for packets */
   socket_mode mode; /* Specify the link direction to enable (RX, TX, both) */
   pkt_header_len header_len;
@@ -1201,6 +1194,8 @@ struct pf_ring_socket {
 
   struct sock *sk;
 
+  char appl_name[64]; /* String that identifies the application bound to the socket */
+
   /* /proc */
   char sock_proc_name[64];       /* /proc/net/pf_ring/<sock_proc_name>             */
   char sock_proc_stats_name[64]; /* /proc/net/pf_ring/stats/<sock_proc_stats_name> */
@@ -1291,10 +1286,6 @@ struct pf_ring_socket {
   /* Indexes (Internal) */
   u_int32_t insert_page_id, insert_slot_id;
 
-  /* Function pointer */
-  do_add_packet_to_ring add_packet_to_ring;
-  do_add_raw_packet_to_ring add_raw_packet_to_ring;
-
   /* Kernel consumer */
   char *kernel_consumer_options, *kernel_consumer_private;
 
