@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2020 - ntop 
+ * (C) 2003-2021 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -58,6 +58,7 @@
 #define MAX_NUM_THREADS        64
 #define DEFAULT_DEVICE     "eth0"
 #define NO_ZC_BUFFER_LEN     9000
+#define BURST_SIZE             32
 
 pfring  *pd;
 int verbose = 0, quiet = 0, num_threads = 1;
@@ -69,7 +70,7 @@ u_int string_id = 1;
 char *out_pcap_file = NULL;
 FILE *match_dumper = NULL;
 u_int8_t do_close_dump = 0, is_sysdig = 0, chunk_mode = 0, check_ts = 0;
-u_int8_t check_seq_ip = 0, asymm_rss = 0;
+u_int8_t check_seq_ip = 0, asymm_rss = 0, burst_mode = 0;
 int num_packets = 0;
 time_t last_ts = 0;
 u_int32_t last_ip = 0;
@@ -302,7 +303,9 @@ void sample_filtering_rules(){
     pfring_toggle_filtering_policy(pd, 0); /* Default to drop */
   }
 
-  if (0) { /* Accolade */
+  if (0) { /* Accolade (Filtering) */
+    /* Default: drop
+     * Pass TCP 10.62.4.239:80 - 10.62.7.8:54830 */
     hw_filtering_rule r = { 0 };
 
     r.rule_family_type = accolade_default;
@@ -329,6 +332,40 @@ void sample_filtering_rules(){
     else
       printf("Rule %d added successfully...\n", r.rule_id );
   }
+
+  if (0) { /* Accolade (Steering) */
+    /* Default: pass
+     * Steer TCP 10.62.4.239:80 - 10.62.7.8:54830 to ring 1 */
+    hw_filtering_rule r = { 0 };
+    u_int8_t ring_id = 1; /* destination ring */
+
+    r.rule_family_type = accolade_default;
+    r.rule_family.accolade_rule.action = accolade_pass;
+
+    pfring_add_hw_rule(pd, &r);
+
+    memset(&r, 0, sizeof(r));
+
+    r.rule_id = 0;
+    r.rule_family_type = accolade_rule;
+    r.rule_family.accolade_rule.action = accolade_pass;
+    r.rule_family.accolade_rule.ip_version = 4;
+    r.rule_family.accolade_rule.protocol = 6;
+    r.rule_family.accolade_rule.src_addr.v4 = ntohl(inet_addr("10.62.4.239"));
+    r.rule_family.accolade_rule.src_addr_bits = 32;
+    r.rule_family.accolade_rule.src_port_low = 80;
+    r.rule_family.accolade_rule.dst_addr.v4 = ntohl(inet_addr("10.62.7.8"));
+    r.rule_family.accolade_rule.dst_addr_bits = 32;
+    r.rule_family.accolade_rule.dst_port_low = 54830;
+
+    r.rule_family.accolade_rule.steer_to_ring = 1;
+    r.rule_family.accolade_rule.ring_id = ring_id;
+
+    if ((rc = pfring_add_hw_rule(pd, &r)) < 0)
+      fprintf(stderr, "pfring_add_hw_rule(id=%d) failed: rc=%d\n", r.rule_id, rc);
+    else
+      printf("Rule %d added successfully...\n", r.rule_id );
+  }
 }
 
 /* ******************************** */
@@ -560,7 +597,7 @@ void print_packet(const struct pfring_pkthdr *h, const u_char *p, u_int8_t dump_
 /* ****************************************************** */
 
 void dummyProcessPacket(const struct pfring_pkthdr *h,
-			 const u_char *p, const u_char *user_bytes) {
+			const u_char *p, const u_char *user_bytes) {
   long threadId = (long)user_bytes;
   u_int8_t dump_match = !!dumper;
 
@@ -662,7 +699,7 @@ void printDevs() {
 /* *************************************** */
 
 void printHelp(void) {
-  printf("pfcount - (C) 2005-2020 ntop.org\n\n");
+  printf("pfcount - (C) 2005-2021 ntop.org\n\n");
   printf("-h              Print this help\n");
   printf("-i <device>     Device name. Use:\n"
 	 "                - ethX@Y for channels\n"
@@ -713,6 +750,7 @@ void printHelp(void) {
   printf("-F              Do not strip CRC/FCS (when not stripped by the adapter)\n");
   printf("-t              Touch payload (to force packet load on cache)\n");
   printf("-M              Packet memcpy (to test memcpy speed)\n");
+  printf("-B              Use the recv burst API (when supported by the module)\n");
   printf("-C <mode>       Work with the adapter in chunk mode (1=chunk API, 2=packet API)\n");
   printf("-T              Check packet timestamps\n");
   printf("-U              Check packet sequential IP as generated by pfsend -b <num IPs>\n");
@@ -796,6 +834,52 @@ void* packet_consumer_thread(void* _id) {
 
 /* *************************************** */
 
+void* burst_consumer_thread(void* _id) {
+  long thread_id = (long)_id;
+  u_int numCPU = sysconf( _SC_NPROCESSORS_ONLN );
+  u_long core_id = thread_id % numCPU;
+  pfring_packet_info packets[BURST_SIZE];
+  struct pfring_pkthdr hdr;
+  int i;
+
+  if((num_threads > 1) && (numCPU > 1)) {
+    if(bind2core(core_id) == 0)
+      if (!quiet)
+        printf("Set thread %lu on core %lu/%u\n", thread_id, core_id, numCPU);
+  }
+
+  memset(&hdr, 0, sizeof(hdr));
+
+  while(1) {
+    int n;
+
+    if(stats->do_shutdown) break;
+
+    if((n = pfring_recv_burst(pd, packets, BURST_SIZE, wait_for_packet)) > 0) {
+      if(stats->do_shutdown) break;
+
+      for (i = 0; i < n; i++) {
+        hdr.len = packets[i].len;
+        hdr.caplen = packets[i].caplen;
+        hdr.extended_hdr.pkt_hash = packets[i].hash;
+        hdr.extended_hdr.rx_direction = 1;
+        hdr.extended_hdr.timestamp_ns = 0;
+        hdr.ts.tv_sec = packets[i].ts.tv_sec;
+        hdr.ts.tv_usec = packets[i].ts.tv_usec;
+
+        dummyProcessPacket(&hdr, packets[i].data, (u_char*)thread_id);
+      }
+
+    } else {
+      if(wait_for_packet == 0) sched_yield();
+    }
+  }
+
+  return(NULL);
+}
+
+/* *************************************** */
+
 void* chunk_consumer_thread(void* _id) {
   long thread_id = (long)_id;
   u_int numCPU = sysconf( _SC_NPROCESSORS_ONLN );
@@ -965,7 +1049,7 @@ int main(int argc, char* argv[]) {
   startTime.tv_sec = 0;
   thiszone = gmt_to_local(0);
 
-  while((c = getopt(argc,argv,"hi:c:C:Fd:H:Jl:Lv:ae:n:w:o:p:qb:rg:u:mtsSx:f:z:N:MRTUK:")) != '?') {
+  while((c = getopt(argc,argv,"Bhi:c:C:Fd:H:Jl:Lv:ae:n:w:o:p:qb:rg:u:mtsSx:f:z:N:MRTUK:")) != '?') {
     if((c == 255) || (c == -1)) break;
 
     switch(c) {
@@ -982,6 +1066,9 @@ int main(int argc, char* argv[]) {
     case 'c':
       clusterId = atoi(optarg);
       break;
+    case 'B':
+      burst_mode = 1;
+      break;
     case 'C':
       chunk_mode = atoi(optarg);
       break;
@@ -1286,6 +1373,8 @@ int main(int argc, char* argv[]) {
 
     if (chunk_mode == 1) {
       chunk_consumer_thread(0);
+    } else if (burst_mode && pd->recv_burst) {
+      burst_consumer_thread(0);
     } else {
       pfring_loop(pd, dummyProcessPacket, (u_char*)NULL, wait_for_packet);
     }
