@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2020 - ntop 
+ * (C) 2003-2021 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -221,7 +221,7 @@ void sigproc(int sig) {
 /* *************************************** */
 
 void printHelp(void) {
-  printf("pfsend - (C) 2011-2020 ntop.org\n");
+  printf("pfsend - (C) 2011-2021 ntop.org\n");
   printf("Replay synthetic traffic, or a pcap, or a packet in hex format from standard input.\n\n"); 
   printf("pfsend -i out_dev [-a] [-f <.pcap file>] [-g <core_id>] [-h]\n"
          "       [-l <length>] [-n <num>] "
@@ -407,6 +407,7 @@ int main(int argc, char* argv[]) {
       break;
     case 'D':
       inet_aton(optarg, &dstaddr);
+      reforge_ip = 1;
       break;
     case 'h':
       printHelp();
@@ -479,6 +480,7 @@ int main(int argc, char* argv[]) {
       break;
     case 'S':
       inet_aton(optarg, &srcaddr);
+      reforge_ip = 1;
       break;
     case 'd':
       daemon_mode = 1;
@@ -616,7 +618,11 @@ int main(int argc, char* argv[]) {
 	struct packet *p;
 	int rc = pcap_next_ex(pt, &h, (const u_char **) &pkt);
 
-	if(rc <= 0) break;
+	if(rc <= 0) {
+          if (rc == PCAP_ERROR)
+            pcap_perror(pt, "" /* prefix */);
+	  break;
+	}
         
         num_orig_pcap_pkts++;
         if ((num_orig_pcap_pkts-1) < ip_offset) continue;
@@ -671,6 +677,9 @@ int main(int argc, char* argv[]) {
           avg_send_len += plen;
 	  num_pcap_pkts++;
 
+          if(num_to_send > 0 && num_pcap_pkts >= num_to_send)
+            break;
+
 	} else {
 	  printf("Not enough memory\n");
 	  break;
