@@ -1,5 +1,5 @@
 /*
- * (C) 2018-2020 - ntop
+ * (C) 2018-2021 - ntop
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -51,15 +51,21 @@
 #define ALARM_SLEEP 1
 #define DEFAULT_DEVICE "eth0"
 
+#ifdef HAVE_NDPI
+#define PRINT_NDPI_INFO /* Note: this requires linking the nDPI library */
+#include "ndpi_api.h"
+#endif
+
 pfring *pd = NULL;
 pfring_ft_table *ft = NULL;
 int bind_core = -1;
 int bind_time_pulse_core = -1;
 u_int8_t quiet = 0, verbose = 0, stats_only = 0;
 u_int8_t time_pulse = 0, enable_l7 = 0, do_shutdown = 0;
-u_int64_t num_pkts = 0;
-u_int64_t num_bytes = 0;
-u_int64_t num_flows = 0;
+u_int64_t num_pkts = 0, num_bytes = 0, num_flows = 0;
+#ifdef PRINT_NDPI_INFO
+u_int8_t enable_l7_extra = 0;
+#endif
 
 volatile u_int64_t *pulse_timestamp;
 
@@ -212,7 +218,7 @@ const char *status_to_string(pfring_ft_flow_status status) {
 void processFlow(pfring_ft_flow *flow, void *user){
   pfring_ft_flow_key *k;
   pfring_ft_flow_value *v;
-  char buf1[32], buf2[32], buf3[32], buf4[32], buf5[32];
+  char buf1[64], buf2[64], buf3[32], buf4[32], buf5[32];
   char *ip1, *ip2;
 
   k = pfring_ft_flow_get_key(flow);
@@ -268,6 +274,15 @@ void processFlow(pfring_ft_flow *flow, void *user){
     case 91:
       if (v->l7_metadata.tls.serverName != NULL)
         printf(", hostName: %s", v->l7_metadata.tls.serverName);
+#ifdef PRINT_NDPI_INFO
+      if (enable_l7_extra) {
+        struct ndpi_flow_struct *ndpi_flow = pfring_ft_flow_get_ndpi_handle(flow);
+        if (ndpi_flow->protos.tls_quic_stun.tls_quic.ja3_server[0] != '\0')
+          printf(", ja3s: '%s'", ndpi_flow->protos.tls_quic_stun.tls_quic.ja3_server);
+        if (ndpi_flow->protos.tls_quic_stun.tls_quic.ja3_client[0] != '\0')
+          printf(", ja3c: '%s'", ndpi_flow->protos.tls_quic_stun.tls_quic.ja3_client);
+      }
+#endif
       break;
   }
 
@@ -336,7 +351,7 @@ void print_version(void) {
   pfring_ft_version(version);
  
   printf("PF_RING FT v.%s\n" 
-         "Copyright 2018 ntop.org\n",
+         "Copyright 2018-2021 ntop.org\n",
          version);
       
   rc = pfring_ft_license(system_id, &license_expiration, &maintenance_expiration);
@@ -353,9 +368,9 @@ void print_version(void) {
     if (license_expiration) {
       printf("License Type:  Time-limited License \n");
       printf("Lic. Duration: Until %s [%u days left]\n", ctime_nonl(license_expiration), days_left(license_expiration));
-    } else if (maintenance_expiration > 0) {
-      printf("License Type:  Permanent License \n");
-      if (days_left(maintenance_expiration) <= 0)
+    } else {
+      printf("License Type:  Permanent License\n");
+      if (maintenance_expiration == 0 || days_left(maintenance_expiration) <= 0)
         printf("Maintenance:   Expired\n");
       else
         printf("Maintenance:   Until %s [%u days left]\n", ctime_nonl(maintenance_expiration), days_left(maintenance_expiration));
@@ -366,7 +381,7 @@ void print_version(void) {
 /* *************************************** */
 
 void print_help(void) {
-  printf("ftflow - (C) 2018 ntop.org\n");
+  printf("ftflow - (C) 2018-2021 ntop.org\n");
   printf("Flow processing based on PF_RING FT (Flow Table)\n\n");
   printf("-h              Print this help\n");
   printf("-i <device>     Device name\n");
@@ -377,10 +392,13 @@ void print_help(void) {
   printf("-g <core>       CPU core affinity\n");
   printf("-S <core>       Enable timer thread and set CPU core affinity\n");
   printf("-s <duration>   Enable flow slicing (set timeout to <duration> seconds\n");
-  printf("-H              Ignore hw hash (use with adapters computing asymmetric hash)");
+  printf("-H              Ignore hw hash (use with adapters computing asymmetric hash)\n");
+#ifdef PRINT_NDPI_INFO
+  printf("-E              Enable extra packet dissection in nDPI to extract more metadata\n");
+#endif
   printf("-q              Quiet mode\n");
   printf("-d              Debug mode\n");
-  printf("-t              Print actual stats");
+  printf("-t              Print actual stats\n");
   printf("-v              Verbose (print also raw packets)\n");
   printf("-V              Print version");
 
@@ -401,7 +419,7 @@ int main(int argc, char* argv[]) {
   pthread_t time_thread;
   u_int8_t ignore_hw_hash = 0;
 
-  while ((c = getopt(argc,argv,"c:dg:hHi:p:qvF:s:S:tV7")) != '?') {
+  while ((c = getopt(argc,argv,"c:dEg:hHi:p:qvF:s:S:tV7")) != '?') {
     if ((c == 255) || (c == -1)) break;
 
     switch(c) {
@@ -412,6 +430,12 @@ int main(int argc, char* argv[]) {
     case 'd':
       pfring_ft_debug();
       break;
+#ifdef PRINT_NDPI_INFO
+    case 'E':
+      enable_l7 = 1;
+      enable_l7_extra = 1;
+      break;
+#endif
     case 'g':
       bind_core = atoi(optarg);
       break;
@@ -465,6 +489,11 @@ int main(int argc, char* argv[]) {
   if (enable_l7)
     ft_flags |= PFRING_FT_TABLE_FLAGS_DPI;
 
+#ifdef PRINT_NDPI_INFO
+  if (enable_l7_extra)
+    ft_flags |= PFRING_FT_TABLE_FLAGS_DPI_EXTRA;
+#endif
+
   if (ignore_hw_hash)
     ft_flags |= PFRING_FT_IGNORE_HW_HASH;
 
