@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2020 - ntop 
+ * (C) 2003-2021 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,6 +35,8 @@
 #include <pthread.h>
 #include <sched.h>
 #include <stdio.h>
+#include <sys/inotify.h>
+#include <fcntl.h>
 
 #include "pfring.h"
 #include "pfring_zc.h"
@@ -42,6 +44,8 @@
 
 #include "zutils.c"
 
+#define DEFAULT_CONF_FILE "/etc/cluster/cluster.conf"
+
 #define bitmap64_t(name, n) u_int64_t name[n / 64]
 #define bitmap64_reset(b) memset(b, 0, sizeof(b))
 #define bitmap64_set_bit(b, i)   b[i >> 6] |=  ((u_int64_t) 1 << (i & 0x3F))
@@ -60,6 +64,9 @@
 #define MAX_NUM_APP	       32
 #define IN_POOL_SIZE          256
 
+#define EVENT_SIZE (sizeof(struct inotify_event))
+#define EVENT_BUF_LEN (1024 * (EVENT_SIZE + 16))
+
 pfring_zc_cluster *zc;
 pfring_zc_worker *zw;
 pfring_zc_queue **inzqs;
@@ -98,6 +105,15 @@ u_int32_t n2disk_threads;
 char *vlan_filter = NULL;
 bitmap64_t(allowed_vlans, 1024);
 
+#define MAX_MAP_VLAN_SIZE 4
+u_int8_t map_vlan_size = 0;
+u_int16_t map_vlan[MAX_MAP_VLAN_SIZE];
+
+#ifdef HAVE_PF_RING_FT
+int notify_fd;
+int notify_wd;
+#endif
+
 /* ******************************** */
 
 #ifdef HAVE_PF_RING_FT
@@ -105,6 +121,7 @@ bitmap64_t(allowed_vlans, 1024);
 
 u_int8_t flow_table = 0;
 pfring_ft_table *ft = NULL;
+char *ft_proto_conf = NULL;
 
 void flow_init(pfring_ft_flow *flow, void *user) {
   // Enable this by uncommenting pfring_ft_set_new_flow_callback()
@@ -407,6 +424,15 @@ void sigproc(int sig) {
   do_shutdown = 1;
 
   print_stats();
+
+#ifdef HAVE_PF_RING_FT
+  if (flow_table) {
+    if (ft_proto_conf != NULL) {
+      inotify_rm_watch(notify_fd, notify_wd);
+      close(notify_fd);
+    }
+  }
+#endif
 }
 
 /* *************************************** */
@@ -589,8 +615,10 @@ void printHelp(void) {
          "                 3 - Fan-out (1st) + Round-Robin (2nd, 3rd, ..)\n"
          "                 4 - GTP hash (Inner IP/Port or Seq-Num or Outer IP/Port)\n"
          "                 5 - GRE hash (Inner or Outer IP)\n"
-         "                 6 - Interface X to queue X\n");
+         "                 6 - Interface X to queue X\n"
+         "                 7 - Ethernet type (check 0x8585 type and select the queue, other to queue 0)\n");
   printf("-r <queue>:<dev> Replace egress queue <queue> with device <dev> (multiple -r can be specified)\n");
+  printf("-M <vlans>       Comma-separated list of VLANs to map VLAN to egress queues (-m 7 only)\n");
   printf("-S <core id>     Enable Time Pulse thread and bind it to a core\n");
   printf("-R <nsec>        Time resolution (nsec) when using Time Pulse thread\n"
          "                 Note: in non-time-sensitive applications use >= 100usec to reduce cpu load\n");
@@ -766,6 +794,42 @@ int64_t direct_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue
 
 /* *************************************** */
 
+#define ETH_P_8585         0x8585
+
+int64_t eth_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *in_queue, void *user) {
+  long num_out_queues = (long) user;
+  u_char *data = pfring_zc_pkt_buff_data(pkt_handle, in_queue);
+  struct ethhdr *eh = (struct ethhdr*) data;
+  u_int16_t eth_type = ntohs(eh->h_proto);
+  int64_t idx = 0;
+
+  if (time_pulse) SET_TS_FROM_PULSE(pkt_handle, *pulse_timestamp_ns);
+
+  if (eth_type == ETH_P_8585) {
+    u_int32_t vlan_offset = sizeof(struct ethhdr);
+
+    idx = -1;
+
+    /* Reforge eth type to ETH_P_8021Q */
+    eh->h_proto = htons(ETH_P_8021Q);
+
+    /* Read VLAN ID */
+    if ((vlan_offset + sizeof(struct eth_vlan_hdr)) < pkt_handle->len) {
+      struct eth_vlan_hdr *vh = (struct eth_vlan_hdr *) &data[vlan_offset];
+      u_int16_t i, vlan_id = ntohs(vh->h_vlan_id) & VLAN_VID_MASK;
+
+      for (i = 0; i < map_vlan_size; i++)
+        if (vlan_id == map_vlan[i]) { idx = i+1; break; }
+    }
+
+    if (idx == -1) return idx; /* drop on no match */
+  }
+
+  return idx % num_out_queues;
+}
+
+/* *************************************** */
+
 static int rr = -1;
 
 int64_t rr_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *in_queue, void *user) {
@@ -911,7 +975,7 @@ int main(int argc, char* argv[]) {
   char *user = NULL;
   int num_consumer_queues_limit = 0;
   u_int32_t flags;
-  const char *opt_string = "ab:c:dD:Ef:G:g:hi:l:m:n:N:pr:Q:q:P:R:S:u:wvx:zW:"
+  const char *opt_string = "ab:c:dD:Ef:G:g:hi:l:m:M:n:N:pr:Q:q:P:R:S:u:wvx:zW:"
 #ifdef HAVE_PF_RING_FT
     "TC:O:"
 #endif
@@ -921,7 +985,6 @@ int main(int argc, char* argv[]) {
   ;
 #ifdef HAVE_PF_RING_FT
   char *ft_rules_conf = NULL;
-  char *ft_proto_conf = NULL;
 #endif
 #ifdef HAVE_ZMQ
   pthread_t zmq_thread;
@@ -932,7 +995,13 @@ int main(int argc, char* argv[]) {
 
   start_time.tv_sec = 0;
 
-  if ((argc == 2) && (argv[1][0] != '-')) {
+  if (argc == 1) {
+    if (load_args_from_file(DEFAULT_CONF_FILE, &opt_argc, &opt_argv) != 0) {
+      trace(TRACE_ERROR, "Please specify all mandatory options via cli or configuration file (default path: %s)\n", DEFAULT_CONF_FILE);
+      printHelp();
+      exit(-1);
+    }
+  } else if ((argc == 2) && (argv[1][0] != '-')) {
     if (load_args_from_file(argv[1], &opt_argc, &opt_argv) != 0) {
       trace(TRACE_ERROR, "Unable to read config file %s\n", argv[1]);
       exit(-1);
@@ -946,12 +1015,12 @@ int main(int argc, char* argv[]) {
     if ((c == 255) || (c == -1)) break;
 
     switch (c) {
-    case 'h':
-      printHelp();
-      break;
     case 'a':
       wait_for_packet = 0;
       break;
+    case 'b':
+      pool_size = upper_power_of_2(atoi(optarg));
+      break;
     case 'c':
       cluster_id = atoi(optarg);
       break;
@@ -970,56 +1039,74 @@ int main(int argc, char* argv[]) {
       else
         append_bpf_file_list(&in_bpf_file_list, optarg);
       break;
-    case 'm':
-      hash_mode = atoi(optarg);
+    case 'g':
+      bind_worker_core = atoi(optarg);
       break;
-    case 'n':
-      applications = strdup(optarg);
+    case 'h':
+      printHelp();
       break;
     case 'i':
       device = strdup(optarg);
       break;
-    case 'g':
-      bind_worker_core = atoi(optarg);
-      break;
     case 'l':
       trace_file = fopen(optarg, "w");
       if (trace_file == NULL)
         trace(TRACE_ERROR, "Unable to open log file %s", optarg);
       break;
-    case 'p':
-      print_interface_stats = 1;
+    case 'm':
+      hash_mode = atoi(optarg);
       break;
-    case 'Q':
-      enable_vm_support = 1;
-      vm_sockets = strdup(optarg);
+    case 'M':
+      if (map_vlan_size < MAX_MAP_VLAN_SIZE) {
+        map_vlan[map_vlan_size] = atoi(optarg);
+        trace(TRACE_NORMAL, "Mapping VLAN %u to queue %u\n", map_vlan[map_vlan_size], map_vlan_size+1);
+        map_vlan_size++;
+      }
       break;
-    case 'q':
-      queue_len = upper_power_of_2(atoi(optarg));
-      break;
-    case 'b':
-      pool_size = upper_power_of_2(atoi(optarg));
+    case 'n':
+      applications = strdup(optarg);
       break;
     case 'N':
       n2disk_producer = 1;
       n2disk_threads = atoi(optarg);
       break;
+    case 'p':
+      print_interface_stats = 1;
+      break;
     case 'P':
       pid_file = strdup(optarg);
       break;
+    case 'q':
+      queue_len = upper_power_of_2(atoi(optarg));
+      break;
+    case 'Q':
+      enable_vm_support = 1;
+      vm_sockets = strdup(optarg);
+      break;
     case 'R':
       time_pulse_resolution = atoi(optarg);
       break;
-    case 'u':
-      if (optarg != NULL) hugepages_mountpoint = strdup(optarg);
-      break;
     case 'S':
       time_pulse = 1;
       bind_time_pulse_core = atoi(optarg);
       break;
+    case 'u':
+      if (optarg != NULL) hugepages_mountpoint = strdup(optarg);
+      break;
+    case 'v':
+      trace_verbosity = 3;    
+    break;
     case 'w':
       hw_aggregation = 1;
       break;
+    case 'W':
+      wait_time_sec = atoi(optarg);
+      if(wait_time_sec > 10)
+	wait_time_sec = 10; /* Upper limit */
+      break;      
+    case 'x':
+      vlan_filter = strdup(optarg);
+    break;
     case 'z':
       proc_stats_only = 1;
       break;
@@ -1048,23 +1135,12 @@ int main(int argc, char* argv[]) {
       time_pulse = 1; /* forcing time-pulse to handle rules expiration */
     break;
 #endif
-    case 'v':
-      trace_verbosity = 3;    
-    break;
-    case 'x':
-      vlan_filter = strdup(optarg);
-    break;
-    case 'W':
-      wait_time_sec = atoi(optarg);
-      if(wait_time_sec > 10)
-	wait_time_sec = 10; /* Upper limit */
-      break;      
     }
   }
-  
+ 
   if (device == NULL) printHelp();
   if (cluster_id < 0) printHelp();
-  if (applications == NULL) printHelp();
+  if (applications == NULL && hash_mode != 7) printHelp();
 
   if (vlan_filter
 #ifdef HAVE_PF_RING_FT
@@ -1111,13 +1187,19 @@ int main(int argc, char* argv[]) {
     num_devices = 1;
   }
 
-  app = strtok_r(applications, ",", &app_pos);
-  while (app != NULL && num_apps < MAX_NUM_APP) {
-    instances_per_app[num_apps] = atoi(app);
-    if (instances_per_app[num_apps] == 0) printHelp();
-    num_consumer_queues += instances_per_app[num_apps];
+  if (hash_mode == 7) {
+    instances_per_app[num_apps] = map_vlan_size + 1;
+    num_consumer_queues += instances_per_app[num_apps]; 
     num_apps++;
-    app = strtok_r(NULL, ",", &app_pos);
+  } else {
+    app = strtok_r(applications, ",", &app_pos);
+    while (app != NULL && num_apps < MAX_NUM_APP) {
+      instances_per_app[num_apps] = atoi(app);
+      if (instances_per_app[num_apps] == 0) printHelp();
+      num_consumer_queues += instances_per_app[num_apps];
+      num_apps++;
+      app = strtok_r(NULL, ",", &app_pos);
+    }
   }
 
   if (num_apps == 0) printHelp();
@@ -1242,7 +1324,7 @@ int main(int argc, char* argv[]) {
 
       rc = pfring_zc_create_queue_pool_pair(zc, queue_len, IN_POOL_SIZE, &ext_q, &ext_pool);
 
-      if (rc < 0 || ext_q == NULL || ext_q == NULL) {
+      if (rc < 0 || ext_q == NULL || ext_pool == NULL) {
         trace(TRACE_ERROR, "pfring_zc_create_queue_pool_pair error [%s]\n", strerror(errno));                                             
         pfring_zc_destroy_cluster(zc);
         return -1;                                                                                                           
@@ -1263,7 +1345,7 @@ int main(int argc, char* argv[]) {
 
     rc = pfring_zc_create_queue_pool_pair(zc,
       outdevs[i] == NULL ? queue_len : 1,
-      outdevs[i] == NULL ? IN_POOL_SIZE : 1,
+      outdevs[i] == NULL ? pool_size : 1,
       &ext_q, &ext_pool);
 
     if (rc < 0 || ext_q == NULL || ext_q == NULL) {
@@ -1381,9 +1463,22 @@ int main(int argc, char* argv[]) {
     if (ft_rules_conf != NULL) 
       pfring_ft_load_configuration(ft, ft_rules_conf);
 
-    if (ft_proto_conf != NULL)
+    if (ft_proto_conf != NULL){
       pfring_ft_load_ndpi_protocols(ft, ft_proto_conf);
 
+      notify_fd = inotify_init();
+      if (fcntl(notify_fd, F_SETFL, O_NONBLOCK) < 0) // error checking for fcntl
+        exit(2);
+
+      notify_wd = inotify_add_watch(notify_fd, ft_proto_conf, IN_MODIFY);
+
+      if (notify_wd == -1)
+        trace(TRACE_WARNING, "Could not watch : %s\n", ft_proto_conf);
+      else
+        trace(TRACE_NORMAL, "Watching protos config at: %s\n", ft_proto_conf);
+    }
+
+    /* FT callbacks configuration examples */
     //pfring_ft_set_new_flow_callback(ft, flow_init, NULL);
     //pfring_ft_set_flow_packet_callback(ft, flow_packet_process, NULL);
     //pfring_ft_set_flow_export_callback(ft, flow_free, NULL);
@@ -1426,7 +1521,13 @@ int main(int argc, char* argv[]) {
 
   trace(TRACE_NORMAL, "Running...");
 
-  if (hash_mode == 0 || ((hash_mode == 1 || hash_mode == 4 || hash_mode == 5 || hash_mode == 6) && num_apps == 1)) { /* balancer */
+  if (hash_mode == 0 || 
+      ((hash_mode == 1 || 
+        hash_mode == 4 || 
+        hash_mode == 5 || 
+        hash_mode == 6 || 
+        hash_mode == 7) && 
+       num_apps == 1)) { /* balancer */
 
     switch (hash_mode) {
       case 0: distr_func = rr_distribution_func;
@@ -1452,6 +1553,9 @@ int main(int argc, char* argv[]) {
       case 6: 
         distr_func =  direct_distribution_func;
       break;
+      case 7: 
+        distr_func =  eth_distribution_func;
+      break;
     }
 
     zw = pfring_zc_run_balancer_v2(
@@ -1523,6 +1627,9 @@ int main(int argc, char* argv[]) {
     pfring_zc_destroy_cluster(zc);
     return -1;
   }
+
+  /* Bind also main thread to the worker core */
+  bind2core(bind_worker_core);
   
   if (user != NULL) {
     if (drop_privileges(user) == 0)
@@ -1534,6 +1641,31 @@ int main(int argc, char* argv[]) {
   while (!do_shutdown) {
     sleep(ALARM_SLEEP);
     print_stats();
+#ifdef HAVE_PF_RING_FT
+    if (flow_table) {
+      if (ft_proto_conf != NULL) {
+        int i = 0, length;
+        char buffer[EVENT_BUF_LEN];
+        /* Read buffer*/
+        length = read(notify_fd, buffer, EVENT_BUF_LEN);
+
+        /* Process the events which has occurred */
+        while (i < length) {
+          struct inotify_event *event = (struct inotify_event *)&buffer[i];
+
+          if (event->len) {
+            if (event->mask & IN_MODIFY) {
+              trace(TRACE_NORMAL, "The protos config at %s was modified\n", event->name);
+              //Reload config
+              //FIXX this should be moved inline
+              pfring_ft_load_ndpi_protocols(ft, ft_proto_conf);
+            }
+          }
+          i += EVENT_SIZE + event->len;
+        }
+      }
+    }
+#endif
   }
 
 #ifdef HAVE_ZMQ
