@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2005-2020 - ntop.org
+ * (C) 2005-2021 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 #endif
 
 #include "pfring.h"
+#include "pfring_priv.h"
 #include <net/ethernet.h>
 
 // #define RING_DEBUG
@@ -34,9 +35,11 @@
 #include "pfring_mod_pcap.h"
 
 #ifdef HAVE_AF_XDP
+#ifdef HAVE_PF_RING_ZC
 /* AF_XDP */
 #include "pfring_mod_af_xdp.h"
 #endif
+#endif
 
 #ifdef HAVE_NT
 /* Napatech */
@@ -120,12 +123,14 @@ static pfring_module_info pfring_module_list[] = {
   },
 
 #ifdef HAVE_AF_XDP
+#ifdef HAVE_PF_RING_ZC
   {
     .name = "xdp",
     .open = pfring_mod_af_xdp_open,
     .findalldevs = NULL
   },
 #endif
+#endif
 
 #ifdef HAVE_DAG
   {
@@ -343,6 +348,7 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
 
   /* default */
   if(!mod_found) {
+    errno = ENODEV;
     ring->device_name = strdup(device_name ? device_name : "any");
     if (ring->device_name == NULL) {
       errno = ENOMEM;
@@ -353,7 +359,6 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
   }
 
   if(ret < 0) {
-    errno = ENODEV;
     if (ring->device_name != NULL) free(ring->device_name);
     free(ring);
     return NULL;
@@ -463,7 +468,7 @@ void pfring_shutdown(pfring *ring) {
   if (!ring)
     return;
 
-  ring->is_shutting_down = ring->break_recv_loop = 1;
+  ring->is_shutting_down = ring->break_recv_loop = ring->break_recv_loop_ext = 1;
 
   if(ring->shutdown)
     ring->shutdown(ring);
@@ -516,7 +521,7 @@ int pfring_loop(pfring *ring, pfringProcesssPacket looper,
 #endif
 
   memset(&hdr, 0, sizeof(hdr));
-  ring->break_recv_loop = 0;
+  ring->break_recv_loop = ring->break_recv_loop_ext = 0;
 
   if((! ring)
      || ring->is_shutting_down
@@ -524,7 +529,7 @@ int pfring_loop(pfring *ring, pfringProcesssPacket looper,
      || ring->mode == send_only_mode)
     return -1;
 
-  while(!ring->break_recv_loop) {
+  while(!ring->break_recv_loop_ext) {
     rc = ring->recv(ring, &buffer, 0, &hdr, wait_for_packet);
 
     if(rc < 0)
@@ -574,7 +579,7 @@ void pfring_breakloop(pfring *ring) {
   if(!ring)
     return;
 
-  ring->break_recv_loop = 1;
+  ring->break_recv_loop = ring->break_recv_loop_ext = 1;
 
   if(ring->one_copy_rx_pfring != NULL)
     ring->one_copy_rx_pfring->break_recv_loop = 1;
@@ -665,6 +670,24 @@ recv_next:
 
 /* **************************************************** */
 
+int pfring_recv_burst(pfring *ring, pfring_packet_info *packets, u_int8_t num_packets, 
+                      u_int8_t wait_for_packets) {
+  if (likely(ring
+	     && ring->enabled
+	     && ring->recv_burst
+	     && ring->mode != send_only_mode)) {
+    ring->break_recv_loop = 0;
+    return ring->recv_burst(ring, packets, num_packets, wait_for_packets);
+  }
+
+  if (!ring->enabled)
+    return PF_RING_ERROR_RING_NOT_ENABLED;
+
+  return PF_RING_ERROR_NOT_SUPPORTED;
+}
+
+/* **************************************************** */
+
 int pfring_recv_parsed(pfring *ring, u_char** buffer, u_int buffer_len,
 		       struct pfring_pkthdr *hdr, u_int8_t wait_for_incoming_packet,
 		       u_int8_t level /* 1..4 */, u_int8_t add_timestamp, u_int8_t add_hash) {
