@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2005-2020 - ntop.org
+ * (C) 2005-2021 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -86,22 +86,6 @@
 
 /* ********************************* */
 
-#ifndef likely
-#define likely(x)       __builtin_expect((x),1)
-#endif
-
-#ifndef unlikely
-#define unlikely(x)     __builtin_expect((x),0)
-#endif
-
-/* ********************************* */
-
-/* See also __builtin_prefetch
- * http://gcc.gnu.org/onlinedocs/gcc/Other-Builtins.html */
-#define prefetch(x) __asm volatile("prefetcht0 %0" :: "m" (*(const unsigned long *)x));
-
-/* ********************************* */
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -165,6 +149,17 @@ typedef struct {
 
 /* ********************************* */
 
+typedef struct {
+  u_char *data;
+  struct timeval ts;     /**< Time stamp */
+  u_int16_t caplen;      /**< Packet captured length */
+  u_int16_t len;         /**< Packet length */
+  u_int32_t flags;       /**< Packet flags (PKT_FLAGS_* defined in pf_ring.h) */
+  u_int32_t hash;        /**< Packet hash */
+} pfring_packet_info;
+
+/* ********************************* */
+
 #ifndef BPF_RELEASE
 struct pfring_bpf_program {
   u_int bf_len; 
@@ -213,7 +208,7 @@ struct __pfring {
   u_int8_t zc_device;
   u_int8_t is_shutting_down;
   u_int8_t socket_default_accept_policy;
-  u_int8_t __padding; 
+  u_int8_t break_recv_loop_ext;
 
   u_int32_t rss_mode;
   packet_direction direction; /* Specify the capture direction for packets */
@@ -308,6 +303,7 @@ struct __pfring {
   void      (*flush_tx_packets)             (pfring *);
   int       (*register_zerocopy_tx_ring)    (pfring *, pfring *);
   int       (*recv_chunk)                   (pfring *, void **, pfring_chunk_info *, u_int8_t); 
+  int       (*recv_burst)                   (pfring *, pfring_packet_info *, u_int8_t, u_int8_t); 
   int       (*set_bound_dev_name)           (pfring *, char *);
   int       (*get_metadata)         	    (pfring *, u_char **, u_int32_t *);
   u_int32_t (*get_interface_speed)	    (pfring *);
@@ -406,6 +402,11 @@ struct __pfring {
 
 /* ********************************* */
 
+/* Defines with new features implemented by the API */
+#define PF_RING_API_RECV_BURST
+
+/* ********************************* */
+
 /**
  * This call is used to initialize a PF_RING socket hence obtain a handle of type struct pfring 
  * that can be used in subsequent calls. Note that: 
@@ -503,6 +504,20 @@ int pfring_stats(pfring *ring, pfring_stat *stats);
 int pfring_recv(pfring *ring, u_char** buffer, u_int buffer_len,
 		struct pfring_pkthdr *hdr, u_int8_t wait_for_incoming_packet);
 
+/**
+ * Similart to pfring_recv, this call returns a set of incoming packets, when available, 
+ * up to the specified number. (experimental - not supported by all modules)
+ * @param ring        The PF_RING handle where we perform the check.
+ * @param packets     An array of packet descriptors that will be filled up received packets.
+ *                    A length of 0 indicates to use the zero-copy optimization, when available.
+ * @param num_packets The number of packet descriptors.
+ * @param wait_for_packets If 0 we simply check the packet availability, otherwise the call 
+ *                    is blocked until at lease one packet is available. 
+ * @return            0 in case of no packet being received (non-blocking), the number of 
+ *                    packets in case of success, -1 in case of error.
+ */
+int pfring_recv_burst(pfring *ring, pfring_packet_info *packets, u_int8_t num_packets, u_int8_t wait_for_packets); 
+
 /**
  * Same of pfring_recv(), with additional parameters to force packet parsing.
  * @param ring
