@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2018-2020 - ntop.org
+ * (C) 2018-2021 - ntop.org
  *
  *  http://www.ntop.org/
  *
@@ -24,6 +24,8 @@
 extern "C" {
 #endif
 
+#define FT_API_VERSION 17
+
 typedef void pfring_ft_table;
 typedef void pfring_ft_list;
 typedef void pfring_ft_flow;
@@ -33,13 +35,13 @@ struct ndpi_flow_struct;
 
 /*** enums ***/
 
-typedef enum {
-  PFRING_FT_ACTION_DEFAULT = 0,
-  PFRING_FT_ACTION_FORWARD,
-  PFRING_FT_ACTION_DISCARD,
-  PFRING_FT_ACTION_USER_1,
-  PFRING_FT_ACTION_USER_2
-} pfring_ft_action;
+#define PFRING_FT_ACTION_DEFAULT 0
+#define PFRING_FT_ACTION_FORWARD 1
+#define PFRING_FT_ACTION_DISCARD 2
+#define PFRING_FT_ACTION_USER_1  3
+#define PFRING_FT_ACTION_USER_2  4
+
+typedef u_int8_t pfring_ft_action;
 
 typedef enum { 
   s2d_direction = 0, /**< Source to destination */
@@ -208,6 +210,7 @@ typedef struct {
 
   pfring_ft_ndpi_protocol l7_protocol; /**< nDPI protocol */
   u_int32_t tunnel_type; /**< nDPI tunnel type (ndpi_packet_tunnel) */
+  u_int32_t tunnel_id;   /**< Tunnel ID (if any) */
 
   union {
     struct {
@@ -705,6 +708,14 @@ pfring_ft_version(
   char *version
 );
 
+/**
+ * Get the PF_RING FT API version.
+ * @return The version number as unsigne inte.
+ */
+u_int32_t
+pfring_ft_api_version(
+);
+
 /**
  * Get license info.
  * @param system_id A buffer (32 bytes long) where system id is returned. (out)
