@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2005-2020 - ntop.org
+ * (C) 2005-2021 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@
 #endif
 
 #include "pfring.h"
+#include "pfring_priv.h"
 #include "pfring_utils.h"
 #include "pfring_hw_filtering.h"
 #include "pfring_mod.h"
@@ -116,8 +117,10 @@ int pfring_mod_open_setup(pfring *ring) {
   if(!strcmp(ring->device_name, "none")) {
     /* No binding yet */
     rc = 0;
-  } else /* "any" or "<interface name>" */
+  } else {
+    /* "any" or "<interface name>" */
     rc = pfring_bind(ring, ring->device_name);
+  }
 
   if(rc < 0) {
     close(ring->fd);
@@ -128,7 +131,7 @@ int pfring_mod_open_setup(pfring *ring) {
 			      MAP_SHARED, ring->fd, 0);
 
   if(ring->buffer == MAP_FAILED) {
-    fprintf(stderr, "[PF_RING] mmap() failed: try with a smaller snaplen\n");
+    fprintf(stderr, "[PF_RING] mmap() failed: %s\n", strerror(errno));
     close(ring->fd);
     return -1;
   }
@@ -139,6 +142,7 @@ int pfring_mod_open_setup(pfring *ring) {
 	   "kernel is %i, libpfring was compiled with %i\n",
 	   ring->slots_info->version, RING_FLOWSLOT_VERSION);
     close(ring->fd);
+    errno = EINVAL;
     return -1;
   }
   memSlotsLen = ring->slots_info->tot_mem;
@@ -153,7 +157,7 @@ int pfring_mod_open_setup(pfring *ring) {
 			      MAP_SHARED, ring->fd, 0);
 
   if(ring->buffer == MAP_FAILED) {
-    fprintf(stderr, "[PF_RING] mmap() failed");
+    fprintf(stderr, "[PF_RING] mmap() failed: %s\n", strerror(errno));
     close(ring->fd);
     return -1;
    }
@@ -181,6 +185,7 @@ int pfring_mod_open_setup(pfring *ring) {
   if(ring->slot_header_len == (u_int16_t)-1) {
     fprintf(stderr, "[PF_RING] ring failure (pfring_get_slot_header_len)\n");
     close(ring->fd);
+    errno = EINVAL;
     return -1;
   }
 
@@ -373,10 +378,18 @@ int pfring_mod_bind(pfring *ring, char *device_name) {
       vlan_dot[0] = '\0';
       vlan_id = atoi(&vlan_dot[1]);
     }
-    
+
     memset(&sa, 0, sizeof(sa));
     sa.sa_family = PF_RING;
-    snprintf(sa.sa_data, sizeof(sa.sa_data), "%s", elem);
+  
+    if (strlen(elem) > sizeof(sa.sa_data))
+      return(PF_RING_ERROR_BAD_IFNAME);
+
+    memcpy(sa.sa_data, elem, strlen(elem));
+
+    /* Terminate string, unless it's 'sa_data size' len (handled by the kernel) */
+    if (strlen(elem) < sizeof(sa.sa_data))
+      sa.sa_data[strlen(elem)] = '\0';
 
     rc = bind(ring->fd, (struct sockaddr *)&sa, sizeof(sa));
     
