@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2020 - ntop.org
+ * (C) 2020-2021 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,6 +14,8 @@
 #ifndef _PFRING_MOD_AF_XDP_H_
 #define _PFRING_MOD_AF_XDP_H_
 
+#ifdef HAVE_PF_RING_ZC
+
 int pfring_mod_af_xdp_open(pfring *ring);
 void pfring_mod_af_xdp_close(pfring *ring);
 int pfring_mod_af_xdp_stats(pfring *ring, pfring_stat *stats);
@@ -29,4 +31,6 @@ int pfring_mod_af_xdp_get_bound_device_address(pfring *ring, u_char mac_address[
 int pfring_mod_af_xdp_get_bound_device_ifindex(pfring *ring, int *if_index);
 u_int8_t pfring_mod_af_xdp_get_num_rx_channels(pfring *ring);
 
+#endif /* HAVE_PF_RING_ZC */
+
 #endif /* _PFRING_MOD_AF_XDP_H_ */
