@@ -1,7 +1,7 @@
 /*
  * PF_RING ZC API
  *
- * (C) 2013-2020 - ntop.org
+ * (C) 2013-2021 - ntop.org
  *
  */
 
@@ -18,8 +18,10 @@
 #include <sys/types.h>
 #include <linux/pf_ring.h> /* needed for hw_filtering_rule */
 
-#define PF_RING_ZC_ENABLE_VM_SUPPORT         (1 << 0) /**< pfring_zc_create_cluster() flag: enable KVM support (memory is rounded up to power of 2 thus it allocates more memory!) */
+/* pfring_zc_create_cluster flags */
+#define PF_RING_ZC_ENABLE_VM_SUPPORT          (1 << 0) /**< pfring_zc_create_cluster() flag: enable KVM support (memory is rounded up to power of 2 thus it allocates more memory!) */
 
+/* pfring_zc_open_device flags */
 #define PF_RING_ZC_DEVICE_ASYMMETRIC_RSS      (1 <<  0) /**< pfring_zc_open_device() flag: use asymmetric hw RSS for multiqueue devices. */
 #define PF_RING_ZC_DEVICE_FIXED_RSS_Q_0       (1 <<  1) /**< pfring_zc_open_device() flag: redirect all traffic to the first hw queue. */
 #define PF_RING_ZC_DEVICE_SW_TIMESTAMP        (1 <<  2) /**< pfring_zc_open_device() flag: compute sw timestamp (please note: this adds per-packet overhead). */
@@ -55,6 +57,13 @@
 #define PF_RING_ZC_BUILTIN_GTP_HASH_FLAGS_GTPC (1 << 2) /**< pfring_zc_builtin_gtp_hash flags: GTP-C */
 #define PF_RING_ZC_BUILTIN_GTP_HASH_FLAGS_GTPU (1 << 3) /**< pfring_zc_builtin_gtp_hash flags: GTP-U */
 
+/* New API features defines */
+#define PF_RING_ZC_API_CLUSTER_INFO
+
+/* Misc defines */
+#define PF_RING_ZC_SEND_PKT_MULTI_MAX_QUEUES 64 /**< pfring_zc_send_pkt_multi: max number of queues in queues_mask */
+#define PF_RING_ZC_BUFFER_HEAD_ROOM          64
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -102,7 +111,7 @@ typedef struct {
 } pfring_zc_pkt_buff;
 
 /**
- * Buffer handle. 
+ * Queue info.
  */
 typedef struct {
   u_int32_t buffer_len;   /**< Max packet length. */
@@ -110,7 +119,24 @@ typedef struct {
 } pfring_zc_queue_info;
 
 /**
- * Return the pointer to the actual packet data.
+ * Cluster required resources info.
+ */
+typedef struct {
+  u_int64_t total_memory;    /**< Total amount of allocated memory. */
+  u_int32_t buffer_len;      /**< Max packet length. */
+  u_int32_t real_buffer_len; /**< Real size of allocated buffers (including head room and padding). */
+} pfring_zc_cluster_info;
+
+/**
+ * Cluster memory info.
+ */
+typedef struct {
+  void *base_addr;   /**< Allocated memory base address. */
+  u_int64_t size;    /**< Total amount of allocated memory. */
+} pfring_zc_cluster_mem_info;
+
+/**
+ * Return the pointer to the actual packet data (same as pfring_zc_pkt_buff_data_from_cluster).
  * @param pkt_handle The buffer handle.
  * @param queue      Any queue from the cluster (e.g. the queue from which the packet is arrived or destined).
  * @return           The pointer on success, NULL otherwise.
@@ -121,6 +147,31 @@ pfring_zc_pkt_buff_data(
   pfring_zc_queue *queue
 );
 
+/**
+ * Return the pointer to the actual packet data (same as pfring_zc_pkt_buff_data).
+ * @param pkt_handle The buffer handle.
+ * @param cluster    The cluster handle.
+ * @return           The pointer on success, NULL otherwise.
+ */
+u_char *
+pfring_zc_pkt_buff_data_from_cluster(
+  pfring_zc_pkt_buff *pkt_handle,
+  pfring_zc_cluster *cluster
+);
+
+/**
+ * Return the buffer handle given a pointer to a actual packet data.
+ * This is the reverse of pfring_zc_pkt_buff_data.
+ * @param data    The packet data pointer.
+ * @param cluster The cluster handle.
+ * @return        The buffer handle on success, NULL otherwise.
+ */
+pfring_zc_pkt_buff *
+pfring_zc_pkt_data_buff(
+	u_char *data,
+	pfring_zc_cluster *cluster
+);
+
 /**
  * Remove data from the start of a buffer.
  * @param pkt_handle The buffer handle.
@@ -149,6 +200,17 @@ pfring_zc_pkt_buff_push(
   u_int16_t len
 );
 
+/**
+ * Remove data from the start of a buffer.
+ * @param pkt_handle The buffer handle.
+ * @param len The number of bytes to remove.
+ */
+void
+pfring_zc_pkt_buff_pull_only(
+  pfring_zc_pkt_buff *pkt_handle,
+  u_int16_t len
+);
+
 /* **************************************************************************************** */
 
 /**
@@ -176,6 +238,39 @@ pfring_zc_create_cluster(
   u_int32_t flags
 );
 
+/**
+ * Return information about the resources that will be allocated the cluster 
+ * including the max amount of memory.
+ * @param info                 A struct that is filled with the cluster information.
+ * @param buffer_len           The size of each buffer: it must be at least as large as the MTU + L2 header (it will be rounded up to cache line) and not bigger than the page size.
+ * @param metadata_len         The size of each buffer metadata.
+ * @param tot_num_buffers      The total number of buffers to reserve for queues/devices/extra allocations.
+ * @param flags                Optional flags:
+ *                             @code
+ *                             PF_RING_ZC_ENABLE_VM_SUPPORT enable KVM support (memory is rounded up to power of 2 thus it allocates more memory!)
+ *                             @endcode
+ * @return                     0 on success, a negative value otherwise (errno is also set appropriately)
+ */
+int
+pfring_zc_precompute_cluster_settings(
+  pfring_zc_cluster_info *info, /**< Out */ 
+  u_int32_t buffer_len,
+  u_int32_t metadata_len,
+  u_int32_t tot_num_buffers,
+  u_int32_t flags
+);
+
+/**
+ * Return information about allocated resources.
+ * @param cluster The cluster handle.
+ * @param mem_info The returned information.
+ */
+int
+pfring_zc_get_memory_info(
+  pfring_zc_cluster *cluster,
+  pfring_zc_cluster_mem_info *mem_info
+);
+
 /**
  * Get the cluster id. 
  * @param cluster The cluster handle.
@@ -250,7 +345,7 @@ pfring_zc_create_queue(
  * @param pool_len  The number of buffers to reserve for the pool.
  * @param queue     The queue handle on success, NULL otherwise (out)
  * @param pool      The pool handle on success, NULL otherwise (out)
- * @return          0 in success, a negative value otherwise (errno is also set appropriately)
+ * @return          0 on success, a negative value otherwise (errno is also set appropriately)
  */
 int
 pfring_zc_create_queue_pool_pair(
@@ -539,7 +634,7 @@ pfring_zc_create_multi_queue(
  * Send a packet to multiple queues bound to a multi-queue object.
  * @param multi_queue  The multi-queue handle.
  * @param pkt_handle   The pointer to the buffer handle to send. Once a packet has been sent, the buffer handle can be reused or if not longer necessary it must be freed by calling pfring_zc_release_packet_handle().
- * @param queues_mask  The mask with the egress queues where the buffer should be inserted. The LSB indicates the first queue in the multi-queue array.
+ * @param queues_mask  The mask with the egress queues where the buffer should be inserted. The LSB indicates the first queue in the multi-queue array. The max number of queues is defined in PF_RING_ZC_SEND_PKT_MULTI_MAX_QUEUES.
  * @param flush_packet The flag indicating whether this call should flush the enqueued packet, and older packets if any.
  * @return             The number of packet copies enqueued. 
  */
